Package: hbamr
Title: Hierarchical Bayesian Aldrich-McKelvey Scaling via 'Stan'
Version: 2.4.6
Authors@R: 
    person("Jørgen", "Bølstad", , "jorgen.bolstad@stv.uio.no", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7623-5741"))
Description: Perform hierarchical Bayesian Aldrich-McKelvey scaling using Hamiltonian Monte
    Carlo via 'Stan'. Aldrich-McKelvey ('AM') scaling is a method for estimating the ideological 
    positions of survey respondents and political actors on a common scale using positional survey 
    data. The hierarchical versions of the Bayesian 'AM' model included in this package outperform 
    other versions both in terms of yielding meaningful posterior distributions for respondent 
    positions and in terms of recovering true respondent positions in simulations. The package 
    contains functions for preparing data, fitting models, extracting estimates, plotting key 
    results, and comparing models using cross-validation. The original version of the default 
    model is described in Bølstad (2024) <doi:10.1017/pan.2023.18>.
License: GPL (>= 3)
URL: https://jbolstad.github.io/hbamr/
BugReports: https://github.com/jbolstad/hbamr/issues
Depends: R (>= 3.4.0)
Imports: colorspace, dplyr, future, future.apply, ggplot2, loo,
        matrixStats, methods, parallel, plyr, progressr, RColorBrewer,
        Rcpp (>= 1.0.7), RcppParallel (>= 5.1.4), rlang, rstan (>=
        2.26.1), rstantools (>= 2.2.0), stats, tidyr
Suggests: data.table, knitr, rmarkdown
LinkingTo: BH (>= 1.66.0), Rcpp (>= 1.0.7), RcppEigen (>= 0.3.3.9.1),
        RcppParallel (>= 5.1.4), rstan (>= 2.26.1), StanHeaders (>=
        2.26.22)
Biarch: true
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
SystemRequirements: GNU make
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-04-28 21:25:01 UTC; boelstad
Author: Jørgen Bølstad [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7623-5741>)
Maintainer: Jørgen Bølstad <jorgen.bolstad@stv.uio.no>
Repository: CRAN
Date/Publication: 2026-04-28 21:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-29 01:48:09 UTC; unix
Archs: hbamr.so.dSYM

# Generated by roxygen2: do not edit by hand

export(fbam)
export(get_est)
export(get_plot_data)
export(hbam)
export(hbam_cv)
export(plot_by_group)
export(plot_over_self)
export(plot_respondents)
export(plot_stimuli)
export(prep_data)
export(prep_data_cv)
import(RColorBrewer)
import(Rcpp)
import(ggplot2)
import(loo)
import(methods)
import(parallel)
import(stats)
import(tidyr)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(colorspace,darken)
importFrom(dplyr,bind_cols)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future,sequential)
importFrom(future.apply,future_lapply)
importFrom(matrixStats,colLogSumExps)
importFrom(plyr,llply)
importFrom(rlang,.data)
importFrom(rstan,sampling)
importFrom(rstan,sflist2stanfit)
importFrom(rstantools,rstan_config)
useDynLib(hbamr, .registration = TRUE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE)

load("data.rda")

library("dplyr")

## ----load_data, echo = TRUE, eval = FALSE, include = TRUE---------------------
# library("hbamr")
# data(LC1980)
# LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
# self <- LC1980[, 1]
# stimuli <- LC1980[, -1]

## ----show_data, eval = FALSE--------------------------------------------------
# head(stimuli)

## ----echo = FALSE-------------------------------------------------------------
head_stimuli 

## ----prep_data, echo = TRUE, eval = FALSE-------------------------------------
# dat <- prep_data(self, stimuli)

## ----fithbam1, eval = FALSE---------------------------------------------------
# fit_hbam <- hbam(self, stimuli)

## ----fitbam, eval = FALSE, cache = FALSE, message = FALSE, warning = FALSE----
# fit_hbam_mini <- hbam(self, stimuli, model = "HBAM_MINI", allow_miss = 0)

## ----fithbam2, cache = FALSE, eval = FALSE, message = FALSE, warning = FALSE----
# dat <- prep_data(self, stimuli, allow_miss = 0)
# fit_hbam <- hbam(data = dat)

## ----prep_data3, include = FALSE, eval = FALSE--------------------------------
# dat <- prep_data(self, stimuli, allow_miss = 0)

## ----eval = FALSE-------------------------------------------------------------
# fit_fbam_multi <- hbam(self, stimuli, model = "FBAM_MULTI", group_id = self,
#                        sigma_alpha = .8, sigma_mu_alpha = .7,
#                        sigma_beta = .4, sigma_mu_beta = .25)

## ----fitfbam, eval = FALSE----------------------------------------------------
# fit_fbam <- fbam(self, stimuli)

## ----include = TRUE, eval = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"----
# plot_stimuli(fit_hbam)

## ----echo = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"----
knitr::include_graphics("p_stim.svg")

## ----include = TRUE, eval = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"----
# plot_respondents(fit_hbam, n_draws = 10)

## ----echo = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"----
knitr::include_graphics("p_resp.svg")

## ----get_plot_dat, include = TRUE, eval = FALSE, include= FALSE, eval = FALSE----
# get_plot_data(fit_hbam)

## ----plot_alpha, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
# plot_over_self(list(fit_hbam, fit_hbam_mini), "alpha")

## ----echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
knitr::include_graphics("p_alpha.svg")

## ----plot_beta, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
# plot_over_self(list(fit_hbam, fit_hbam_mini), "abs_beta")

## ----echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
knitr::include_graphics("p_abs_beta.svg")

## ----plot_lambda, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
# plot_over_self(list(fit_hbam, fit_hbam_mini), "lambda")

## ----echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
knitr::include_graphics("p_lambda.svg")

## ----plot_chi, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
# plot_over_self(list(fit_hbam, fit_hbam_mini), "chi")

## ----echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"----
knitr::include_graphics("p_chi.svg")

## ----plot_eta, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .67, fig.width = 4.25, fig.align = "center", out.width = "50%"----
# fit_hbam <- hbam(data = dat, extra_pars = "eta")
# plot_over_self(fit_hbam, "eta")

## ----echo = FALSE, fig.align = "center", fig.asp = .67, fig.width = 4.25, fig.align = "center", out.width = "50%"----
knitr::include_graphics("p_eta.svg")

## ----get_est_t, include = TRUE, eval = FALSE----------------------------------
# get_est(fit_hbam, "theta")

## ----echo = FALSE-------------------------------------------------------------
est_theta

## ----get_est_v, include = TRUE, eval = FALSE----------------------------------
# get_est(fit_hbam, "chi", format_orig = TRUE)

## ----echo = FALSE-------------------------------------------------------------
est_chi_orig

## ----eval = FALSE-------------------------------------------------------------
# fit_hbam <- hbam(data = dat, extra_pars = "log_lik")
# loo_hbam <- loo::loo(fit_hbam)

## ----eval = FALSE-------------------------------------------------------------
# library(future)
# plan(multisession, workers = 4)

## ----include = FALSE----------------------------------------------------------
load("elpds.rda")

## ----run_cv_new, eval = FALSE-------------------------------------------------
# kfold_bam <- hbam_cv(self, stimuli, model = "BAM")
# kfold_hbam <- hbam_cv(self, stimuli, model = "HBAM")
# kfold_hbam_nf <- hbam_cv(self, stimuli, model = "HBAM_NF")
# kfold_hbam_multi <- hbam_cv(self, stimuli, group_id = self,
#                             model = "HBAM_MULTI")

## ----eval = FALSE-------------------------------------------------------------
# print(loo::loo_compare(list(BAM = kfold_bam,
#                             HBAM = kfold_hbam,
#                             HBAM_NF = kfold_hbam_nf,
#                             HBAM_MULTI = kfold_hbam_multi)), simplify = FALSE)

## ----echo = FALSE, message = FALSE, warning = FALSE---------------------------
library(loo)
print(loo1, simplify = FALSE)

## ----eval = FALSE-------------------------------------------------------------
# kfold_hbam_mini <- hbam_cv(self, stimuli, model = "HBAM_MINI")

## ----eval = FALSE-------------------------------------------------------------
# kfold_hbam_mini

## ----echo = FALSE-------------------------------------------------------------
kfold_hbam_mini

## ----traceplot_code, eval = FALSE, include = TRUE-----------------------------
# rstan::traceplot(fit_hbam, pars = "theta")

## ----echo = FALSE, fig.align = "center", fig.asp = .5, fig.width = 8, fig.align = "center", out.width = "100%", warning = FALSE, message = FALSE----
knitr::include_graphics("p_trace_theta.svg")

#' Prepare data for a K-fold cross-validation of an HBAM model
#'
#' This function turns data prepared for `hbam()` into a list of K versions, where each version includes a different vector identifying holdout-data.
#'
#' @export
#' @param data A list of data produced by `prep_data()`.
#' @param K An integer above 2, specifying the number of folds to use in the analysis. Defaults to 10.
#' @param seed An integer passed on to `set.seed` before creating the folds to increase reproducibility. Defaults to 1.
#' @return A list of K data objects where each version includes a different vector identifying holdout-data.
#' @examples
#' # Loading and re-coding ANES 1980 data:
#' data(LC1980)
#' LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
#' self <- LC1980[, 1]
#' stimuli <- LC1980[, -1]
#' dat <- prep_data(self, stimuli)
#'
#' # Prepare data for cross-validation:
#' dat_cv <- prep_data_cv(dat, K = 10)
#'

prep_data_cv <- function(data, K = 10, seed = 1) {
  set.seed(seed)
  data$MCMC <- 1
  hh <- loo::kfold_split_stratified(K = K, x = data$ii)
  holdout_k <- matrix(0, nrow = data$N_obs, ncol = K)
  for(i in 1:data$N_obs) holdout_k[i, hh[i]] <- 1
  holdout_k <- split(holdout_k, rep(1:ncol(holdout_k), each = nrow(holdout_k)))
  data_l <- rep(list(data), K)
  for(i in 1:K) {
    data_l[[i]]$holdout <- holdout_k[[i]]
    data_l[[i]]$CV <- 1
  }
  return(data_l)
}

run_para_sims <- function(nreps = 1, J_vals = 4, psi_vals = .89, tau_vals = .65,
                          it = 4000, warm = 1000, draws = 1500, ...) {

  # Create vectors containing settings for all simulations to be run
  vecs <- data.frame(
    J = rep(rep(J_vals, length(psi_vals) * length(tau_vals)), nreps),
    psi = rep(rep(rep(psi_vals, length(tau_vals)), each = length(J_vals)), nreps),
    tau = rep(rep(tau_vals, each = length(J_vals) * length(psi_vals)), nreps)
  )
  vecs$seed <- 1:nrow(vecs)

  # Run simulations in parallel
  res_list <-
    future.apply::future_lapply(1:nrow(vecs),
             function(i){
               results <- try(run_sim(tau = vecs$tau[i], J = vecs$J[i], psi = vecs$psi[i],
                                  chains = 1, it = it, warm = warm, draws = draws,
                                  seed = vecs$seed[i], ...))
               if(inherits(results, "try-error")) {
                 error <- as.character(results)
                 results <- data.frame(rbind(rep(NA, 38)), vecs$seed[i], error)
                 names(results) <- c("N", "J", "nu", "tau", "psi", "cor_mean_spos", "ppos_mod1", "ppos_mod2", "ppos_mod3",
                   "cor_V_true_chi", "vpos_mod1", "vpos_mod2", "vpos_mod3","vpos_mean_mod1", "vpos_mean_mod2", "vpos_mean_mod3",
                   "mod3_median_psi", "mod3_mean_kappa","chi_cov_mod1", "chi_cov_mod1_ex_extr", "chi_cov_mod2", "chi_cov_mod3",
                   "extr_CI_mod1", "extr_CI_mod2", "extr_CI_mod3","rhat_mod1", "rhat_mod2", "rhat_mod3",
                   "rhat_all_mod1", "rhat_all_mod2", "rhat_all_mod3","neff_mod1", "neff_mod2", "neff_mod3","time_mod1", "time_mod2", "time_mod3", "time_it",
                   "seed", "error")
               }
               return(results)
             })

  results_full <- data.table::rbindlist(res_list)
  return(results_full)
}

# Function to run simulations:
run_sim <- function(N = 500, J = 4, nu = 8.4, tau = .57, psi = .89,
                    chains = 4, it = 2000, warm = 1000, draws = 1000, seed = 1, ...) {

  ptm.it <- proc.time()
  dat <- create_data(N = N, J = J, nu = nu, tau = tau, psi = psi, seed = seed, ...)
  thin <- round((it - warm) / (draws / chains))

  ptm <- proc.time()
  mod1_fit <- hbamr::hbam(data = dat, model = "BAM", prep_data = FALSE,
                   chains = chains, cores = chains, refresh = 0,
                   warmup = warm, iter = it, thin = thin, seed = seed)
  t_mod1 <- (proc.time() - ptm)[[3]]

  ptm <- proc.time()
  mod2_fit <- hbamr::hbam(data = dat, model = "HBAM_0", prep_data = FALSE,
                   chains = chains, cores = chains, refresh = 0,
                   warmup = warm, iter = it, thin = thin, seed = seed)
  t_mod2 <- (proc.time() - ptm)[[3]]

  ptm <- proc.time()
  mod3_fit <- hbamr::hbam(data = dat, model = "HBAM", prep_data = FALSE,
                   chains = chains, cores = chains, refresh = 0,
                   warmup = warm, iter = it, thin = thin, seed = seed)
  t_mod3 <- (proc.time() - ptm)[[3]]

  mod1_theta <- rstan::summary(mod1_fit, pars = "theta")[[1]][, 6]
  mod1_chi <- rstan::summary(mod1_fit, pars = "chi")[[1]][, 6]
  mod1_mean_chi <- rstan::summary(mod1_fit, pars = "chi")[[1]][, 1]

  mod2_theta <- rstan::summary(mod2_fit, pars = "theta")[[1]][, 6]
  mod2_chi <- rstan::summary(mod2_fit, pars = "chi")[[1]][, 6]
  mod2_mean_chi <- rstan::summary(mod2_fit, pars = "chi")[[1]][, 1]

  mod3_theta <- rstan::summary(mod3_fit, pars = "theta")[[1]][, 6]
  mod3_chi <- rstan::summary(mod3_fit, pars = "chi")[[1]][, 6]
  mod3_mean_chi <- rstan::summary(mod3_fit, pars = "chi")[[1]][, 1]

  mod3_median_psi <- rstan::summary(mod3_fit, pars = "psi")[[1]][6]
  mod3_mean_kappa <- mean(rstan::summary(mod3_fit, pars = "kappa")[[1]][, 1])

  mod1_extr <- extreme_CIs(mod1_fit)
  mod1_cov <- CI_coverage(mod1_fit, dat)

  mod2_extr <- extreme_CIs(mod2_fit)
  mod2_cov <- CI_coverage(mod2_fit, dat)

  mod3_extr <- extreme_CIs(mod3_fit)
  mod3_cov <- CI_coverage(mod3_fit, dat)

  results <- data.frame(N, J, nu, tau, psi, cor(dat$mean_spos, dat$true_theta),
    cor(mod1_theta, dat$true_theta), cor(mod2_theta, dat$true_theta), cor(mod3_theta, dat$true_theta),
    cor(dat$V, dat$true_chi), cor(mod1_chi, dat$true_chi), cor(mod2_chi, dat$true_chi), cor(mod3_chi, dat$true_chi),
    cor(mod1_mean_chi, dat$true_chi), cor(mod2_mean_chi, dat$true_chi), cor(mod3_mean_chi, dat$true_chi),
    mod3_median_psi, mod3_mean_kappa,
    mean(mod1_cov), mean(mod1_cov[!mod1_extr]), mean(mod2_cov), mean(mod3_cov),
    mean(mod1_extr), mean(mod2_extr), mean(mod3_extr),
    max(rstan::summary(mod1_fit, pars = c("theta", "chi"))$summary[, 10]),
    max(rstan::summary(mod2_fit, pars = c("theta", "chi"))$summary[, 10]),
    max(rstan::summary(mod3_fit, pars = c("theta", "chi"))$summary[, 10]),
    max(rstan::summary(mod1_fit)$summary[, 10]),
    max(rstan::summary(mod2_fit)$summary[, 10]),
    max(rstan::summary(mod3_fit)$summary[, 10]),
    min(rstan::summary(mod1_fit, pars = c("theta", "chi"))$summary[, 9]),
    min(rstan::summary(mod2_fit, pars = c("theta", "chi"))$summary[, 9]),
    min(rstan::summary(mod3_fit, pars = c("theta", "chi"))$summary[, 9]),
    t_mod1, t_mod2, t_mod3, (proc.time() - ptm.it)[[3]], seed, "no")

  names(results) <- c(
    "N", "J", "nu", "tau", "psi", "cor_mean_spos",
    "ppos_mod1", "ppos_mod2", "ppos_mod3",
    "cor_V_true_chi", "vpos_mod1", "vpos_mod2", "vpos_mod3",
    "vpos_mean_mod1", "vpos_mean_mod2", "vpos_mean_mod3",
    "mod3_median_psi", "mod3_mean_kappa",
    "chi_cov_mod1", "chi_cov_mod1_ex_extr", "chi_cov_mod2", "chi_cov_mod3",
    "extr_CI_mod1", "extr_CI_mod2", "extr_CI_mod3",
    "rhat_mod1", "rhat_mod2", "rhat_mod3",
    "rhat_all_mod1", "rhat_all_mod2", "rhat_all_mod3",
    "neff_mod1", "neff_mod2", "neff_mod3",
    "time_mod1", "time_mod2", "time_mod3", "time_it",
    "seed", "error")

  return(results)
}

#' Extract point estimates or other summaries of marginal posterior distributions
#'
#' For objects produced by `hbam()`, this function is a wrapper for `rstan::summary()`. For objects produced by `fbam()` it offers a way to extract point estimates.
#'
#' @export
#' @param object An instance of class `stanfit` produced by `hbam()`, or a list produced by `fbam()`.
#' @param par Character: Name of the parameter type to be extracted. Typically `"theta"` (stimuli positions) or `"chi"` (respondent positions).
#' @param format_orig Logical: Should individual-level parameters be mapped to the original dataset by returning rows of NAs for respondents who were not included in the analysis? Defaults to `FALSE`.
#' @param probs A numeric vector of quantiles of interest for summarizing `stanfit` objects.
#' @param simplify Logical: Should the returned object be simplified by dropping the Monte Carlo standard error and the posterior standard deviation? Defaults to `TRUE`.
#' @param ... Other arguments are passed on to `rstan::summary()` when summarizing `stanfit` objects.
#' @return A tibble containing summaries of marginal posterior distributions. For objects produced by `fbam()`, only maximum a posteriori estimates are returned.

get_est <- function (object, par = "theta", format_orig = FALSE, probs = c(0.025, 0.5, 0.975), simplify = TRUE, ...) {
  if (inherits(object, "stanfit")) {
    out <- as.data.frame(rstan::summary(object, par, probs = probs, ...)[[1]])
    if (simplify == TRUE) { out <- out[, -c(2, 3)] }
    data <- object@.MISC$hbam_data
  } else {
    if (inherits(object, "list")) {
      if (par %in% names(object$par)) {
        out <- matrix(object$par[which(names(object$par) == par)], ncol = 1)
      } else {
        out <- matrix(object$par[startsWith( names(object$par), paste0(par, "[") )], ncol = 1)
      }
      colnames(out) <- par
      data <- object$hbam_data
    }
  }
  if (format_orig) {
    if (is.null(data) || !inherits(data, "hbam_data")) {
      stop("Could not find the data used for fitting within the supplied object. You need to supply an object produced by hbam() or fbam().")
    }
    if (nrow(out) == data$N) {
      out_long <- matrix(NA, ncol = ncol(out), nrow = length(data$keep))
      out_long[data$keep, ] <- unlist(out[, ])
      colnames(out_long) <- colnames(out)
      out <- out_long
    }
  }
  out <- dplyr::as_tibble(out)

  return(out)
}

#' Fit an FBAM model using optimization
#'
#' Fit a simplified Bayesian Aldrich-McKelvey model with fixed hyperparameters using optimization via `rstan`. Users may replace the default priors by supplying their own values for the hyperparameters.
#'
#' @export
#' @param self An optional numerical vector of N ideological self-placements. Any missing data must be coded as NA. If this argument is not supplied (either here or in a previous call to `prep_data()`), respondent positions will not be estimated. If the data have been prepared in advance via the `prep_data()` function, the argument supplied here will be ignored.
#' @param stimuli An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param model Character: Name of the model to be used. Defaults to FBAM. The available options are the three models with "FBAM" in their name. See the documentation for the `hbam()` function for descriptions of the models.
#' @param allow_miss Integer specifying how many missing stimulus positions to be accepted for an individual still to be included in the analysis. This argument will not be used if the data have been prepared in advance via the `prep_data()` function. Defaults to 2.
#' @param req_valid Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is `req_valid = J - allow_miss`, but if specified, `req_valid` takes precedence. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param req_unique Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is `req_unique = 2`. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param data List of data that have been prepared in advance via the `prep_data()` function. Not required if the arguments `self` and `stimuli` are provided.
#' @param group_id Vector of length N identifying which group each respondent belongs to. The format can be factor, character, integer, or numeric. Respondents with NAs on `group_id` will be dropped when `group_id` is supplied. These data are only required by models with `"MULTI"` in their name and will be ignored when fitting other models.
#' @param seed A positive integer specifying an optional seed for reproducibility. If this argument is not supplied, a random seed will be generated and the function will produce slightly different results on each run.
#' @param sigma_alpha A positive numeric value specifying the standard deviation of the prior on the shift parameters in the FBAM model, or the standard deviation of the parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to B / 5, where B measures the length of the survey scale as the number of possible placements on one side of the center.
#' @param sigma_beta A positive numeric value specifying the standard deviation of the prior on the logged stretch parameters in the FBAM model, or the standard deviation of the logged parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to .3.
#' @param sigma_mu_alpha A positive numeric value specifying the standard deviation of the prior on the group-means of the shift parameters in MULTI-type models. Defaults to B / 10.
#' @param sigma_mu_beta A positive numeric value specifying the standard deviation of the prior on the group-means of the logged stretch parameters in MULTI-type models. Defaults to .2.
#' @param ... Arguments passed to `rstan::optimizing()`.
#' @return A list produced by `rstan::optimizing()`.
#'
#' @examples
#' \donttest{
#' # Loading ANES 2012 data:
#' data(LC2012)
#'
#' # Making a small subset of the data for illustration:
#' self <- LC2012[1:1000, 2]
#' stimuli <- LC2012[1:1000, -c(1:2)]
#'
#' # Fitting the FBAM model:
#' fit_fbam <- fbam(self, stimuli)
#'
#' # Obtaining point estimates for the latent stimulus positions:
#' theta_est <- get_est(fit_fbam, par = "theta")
#' }

fbam <- function(self = NULL, stimuli = NULL, model = "FBAM", allow_miss = 2, req_valid = NA,
                 req_unique = 2, group_id = NULL, data = NULL,
                 seed = sample.int(.Machine$integer.max, 1),
                 sigma_alpha = NULL, sigma_beta = .3,
                 sigma_mu_alpha = NULL, sigma_mu_beta = .2, ...) {
  if (!model %in% c("FBAM", "FBAM_MULTI", "FBAM_MULTI_NF")) { stop(paste(model, "is not a valid model choice for optimization.")) }
  if (!is.null(data) & (!is.null(self) | !is.null(stimuli))) { message("Note: When pre-prepared data are supplied, other data arguments will be ignored.") }
  if (is.null(data) & (is.null(self) | is.null(stimuli))) { message("Note: Required data not supplied.") }
  if (is.null(data)) {
    dat <- hbamr::prep_data(self, stimuli, allow_miss = allow_miss, req_valid = req_valid, req_unique = req_unique, group_id = group_id)
  } else {
    if (inherits(data, "hbam_data")) {
      dat <- data
    } else {
      stop("Supplied data is not of hbam_data class. Please use the prep_data() function from the hbamr package to prepare the data.")
    }
  }
  if (grepl("MULTI", model) & dat$G <= 1) { stop("No group_id supplied for MULTI-type model.") }
  if (!grepl("MULTI", model) & dat$G > 1) { message("Note: The supplied group_id will not be used as the chosen model is not a MULTI-type model.") }

  if (is.null(sigma_alpha)) { sigma_alpha <- dat$B / 5 }
  if (is.null(sigma_mu_alpha)) { sigma_mu_alpha <- dat$B / 10 }
  dat$sigma_alpha_fixed <- sigma_alpha
  dat$sigma_mu_alpha <- sigma_mu_alpha
  dat$sigma_beta_fixed <- sigma_beta
  dat$sigma_mu_beta <- sigma_mu_beta
  dat$MCMC <- 0

  dat <- add_model_features(dat, model)

  set.seed(seed)
  out <- rstan::optimizing(stanmodels[[1]], data = dat, init = inits_omni(1, dat), seed = seed, ...)
  out$hbam_data <- dat
  return(out)
}

#' Hierarchical Bayesian Aldrich-McKelvey Scaling via Stan
#'
#' @description Fit hierarchical Bayesian Aldrich-McKelvey (HBAM) models using a form of Hamiltonian Monte Carlo via Stan. Aldrich-McKelvey (AM) scaling is a method for estimating the ideological positions of survey respondents and political actors on a common scale using positional survey data. The hierarchical versions of the Bayesian AM model included in this package outperform other versions both in terms of yielding meaningful posterior distributions for respondent positions and in terms of recovering true respondent positions in simulations. The package contains functions for preparing data, fitting models, extracting estimates, plotting key results, and comparing models using cross-validation.
#'
#' @author Jørgen Bølstad
#'
#' @seealso \itemize{\item \url{https://jbolstad.github.io/hbamr/}}
#'
#' @docType package
#' @name hbamr-package
#' @aliases hbamr
#' @useDynLib hbamr, .registration = TRUE
#' @import methods stats
#' @import Rcpp ggplot2 RColorBrewer parallel tidyr loo
#' @importFrom rstan sampling sflist2stanfit
#' @importFrom matrixStats colLogSumExps
#' @importFrom future plan multisession sequential
#' @importFrom future.apply future_lapply
#' @importFrom plyr llply
#' @importFrom dplyr bind_cols
#' @importFrom rlang .data
#' @importFrom colorspace darken
#' @importFrom RcppParallel RcppParallelLibs
#' @importFrom rstantools rstan_config
#'
#' @references
#' - Bølstad, Jørgen (2024). Hierarchical Bayesian Aldrich-McKelvey Scaling. \emph{Political Analysis}. 32(1): 50–64. \doi{10.1017/pan.2023.18}.
#' - Stan Development Team (2024). RStan: the R interface to Stan. \url{https://mc-stan.org}.
#'
NULL

get_post_mode_x <- function(d) {dd <- density(d); dd$x[which.max(dd$y)]}
get_post_mode_y <- function(d) {dd <- density(d); dd$y[which.max(dd$y)]}

extreme_CIs <- function(object, prob = c(.025, .975), par = "chi", check_fac = 1.5) {
  draws <- as.data.frame(rstan::extract(object, pars = "chi")$chi)
  draws <- tidyr::pivot_longer(draws, starts_with("V"), values_to = "chi")
  qtile <- quantile(draws$chi, prob)
  prange <- qtile[2] - qtile[1]
  CI <- rstan::summary(object, pars = par)[[1]][, c(4, 8)]
  ranges <- CI[, 2] - CI[, 1]
  rm(draws)
  return(ranges > prange * check_fac)
}

out_of_pop_dist <- function(object, prob = c(.025, .975), par = "chi", check_fac = 2) {
  mean_median<- rstan::summary(object, pars = par)[[1]][, c(1, 6)]
  draws <- as.data.frame(rstan::extract(object, pars = "chi")$chi)
  draws <- tidyr::pivot_longer(draws, starts_with("V"), values_to = "chi")
  median_all <- median(draws$chi)
  mad_all <- mad(draws$chi)
  range <- c(median_all - mad_all * check_fac, median_all + mad_all * check_fac)
  rm(draws)
  return(data.frame(mean_outof_range = !(mean_median[, 1] > range[1] & mean_median[, 1] < range[2]),
                    median_outof_range = !(mean_median[, 2] > range[1] & mean_median[, 2] < range[2])))
}

CI_coverage <- function(object, dat, par = "chi") {
  CI <- rstan::summary(object, pars = par)[[1]][, c(4, 8)]
  median_est <- rstan::summary(object, pars = par)[[1]][, 6]
  CI <- (CI - mean(median_est)) / sd(median_est)
  truevals <- dat[[paste("true_", par, sep = "")]]
  truevals <- (truevals - mean(truevals)) / sd(truevals)
  in_CI <- truevals > CI[, 1] & truevals < CI[, 2]
  return(in_CI)
}

#' Plot posterior densities of parameter averages by group
#'
#' Plot posterior densities of group summaries of individual parameters. The respondents can be grouped by any categorical variable and the function works whether the fitted model is of "MULTI"-type or not.
#'
#' @export
#' @param object An instance of class `stanfit` produced by `hbam()`.
#' @param par Character: Name of the parameter to be plotted. One of the following: `"alpha"`, `"beta"`, `"abs_beta"`, `"lambda"`, or `"chi"`. Defaults to `"abs_beta"`, which means the absolute values of the draws for beta will be used. Further individual-level parameters like `"eta"` can be specified if these have been passed to `hbam()` via the argument `extra_pars` when fitting the model. (Note that homoskedastic models have no `"eta"` parameters and "NF"-type models have no `"lambda"` or `"kappa"` parameters.)
#' @param group_id An optional vector that will be used to split the respondents into groups. The vector must either be as long as the number of rows in the original dataset, or as long as the number of respondents included in the analysis. If a `group_id` was previously supplied to `prep_data()` or `hbam()` and if no `group_id` is supplied here, the default is to use the existing `group_id`. If a `group_id` is supplied here, it will be used instead of any previously supplied vector. The `group_id` supplied here does not have to coincide with the `group_id` used to fit a "MULTI"-type model: Any vector that can be used to group the respondents is allowed.
#' @param ascending_means Logical: Should the groups be placed in ascending order based on their posterior means (`TRUE`) or should they be ordered based on their names (`FALSE`)? Defaults to `TRUE`.
#' @param fill Fill color. Passed on to `ggplot2::geom_density()`.
#' @param color Color of outer lines. Passed on to `ggplot2::geom_density()`.
#' @param alpha Number in \[0,1\]: Inverse level of transparency.
#' @param ncol Number of columns. The default uses a formula to have approximately ten subplots per column.
#' @return A `ggplot` object.
#'

plot_by_group <- function(object, par = "abs_beta", group_id = NULL, ascending_means = TRUE, fill = "#2166AC", color = "#053061", alpha = .5, ncol = max(1, round(length(unique(group_id))/10))) {
  data <- object@.MISC$hbam_data
  if (is.null(data) || !inherits(data, "hbam_data")) {
    stop("Could not find the data used for fitting within the supplied object. You need to supply an object produced by hbam() or fbam().")
  }

  if (is.null(group_id)) {
    if (is.null(data$ggfac)) {
      stop("You need to supply a group_id to plot_by_group() if no group_id was previously supplied to hbam() or prep_data().")
    }
    group_id <- data$ggfac
  } else {
    if (!(length(group_id) == data$N_orig | length(group_id) == data$N)) {
      stop("The group_id must either be the same length as the original data or the number of respondents included in the analysis.") }
    if (length(group_id) == data$N_orig) {
      group_id <- group_id[data$keep]
    }
  }

  if (par == "abs_beta") {
    draws <- abs(rstan::extract(object, pars = "beta")[["beta"]])
  } else {
    draws <- rstan::extract(object, pars = par)[[par]]
  }
  means <- apply(draws, 1, function(x) tapply(x, group_id, mean))
  mean_means <- apply(means, 1, mean)
  group_names <- attributes(means)$dimnames[[1]]
  means <- data.frame(group_id = group_names, means)
  if (ascending_means) {
    means$group_id <- factor(means$group_id, levels = group_names[order(mean_means)])
  } else {
    means$group_id <- factor(means$group_id, levels = group_names)
  }
  means_long <- tidyr::pivot_longer(means, !group_id, names_to = "draw_no")
  lim <- quantile(means_long$value, probs = c(.001, .999))
  plot <- ggplot2::ggplot(means_long, aes(x = .data$value)) + ggplot2::geom_density(fill = fill, color = color, alpha = alpha, na.rm = TRUE) +
    ggplot2::xlim(lim[1], lim[2]) + ggplot2::labs(x = par, y = "Posterior density") + ggplot2::facet_wrap(~group_id, ncol = ncol)
  return(plot)
}

#' 2012 Liberal-Conservative Scales
#'
#' Liberal-Conservative 7-point scales from the 2012 National Election Study. Includes
#' (in order) original case id, self-placement, and rankings of Obama, Romney, Democratic Party,
#' Republican party. Missing values are coded as NA.
#'
#' @docType data
#'
#' @usage data(LC2012)
#'
#' @keywords datasets
#'
#' @source American National Election Studies.
#'
#' @examples
#' data(LC2012)
#' head(LC2012)
"LC2012"

#' Prepare data to fit an HBAM or FBAM model
#'
#' This function prepares data to fit a hierarchical Bayesian Aldrich-McKelvey (HBAM) model. It can be run ahead of fitting the models, or it can be run implicitly as part of a single function call to fit the models using `hbam()` or `fbam()`. It applies a set of inclusion criteria, performs any necessary data transformation, and returns a list of data suited for sampling in `rstan`. The data provided to `prep_data()` can be centered, but they do not have to be: The function will detect un-centered data and attempt to center these automatically, assuming that the highest and lowest observed values in the data mark the extremes of the scale.
#'
#' @export
#' @param self An optional numerical vector of N ideological self-placements. Any missing data must be coded as NA. If this argument is not supplied, respondent positions will not be estimated.
#' @param stimuli An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA.
#' @param prefs An N × J matrix of numerical stimulus ratings or preference scores. These data are only required by the `"HBAM_R_MINI"` model and will be ignored when fitting other models.
#' @param allow_miss Integer specifying how many missing stimulus positions should be accepted for an individual still to be included in the analysis. Defaults to 2.
#' @param req_valid Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is `req_valid = J - allow_miss`, but if specified, `req_valid` takes precedence.
#' @param req_unique Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is `req_unique = 2`.
#' @param B Scalar specifying the upper bound of the survey scale after centering. If not supplied, this information will be inferred from the data.
#' @param group_id Vector of length N identifying which group each respondent belongs to. The format can be factor, character, integer, or numeric. Respondents with NAs on `group_id` will be dropped when `group_id` is supplied. These data are only required by models with `"MULTI"` in their name and will be ignored when fitting other models.
#' @param quiet Logical: Should information about the data be printed to the console? Defaults to `FALSE`.
#' @return A list of data to be used by `hbam()` or `fbam()`. The returned list includes the logical vector `keep`, which identifies the rows in the original data that have been kept for further analysis. The stimuli data are stored in a vector as a long-form sparse matrix. If the stimuli data include column-names, these will be preserved for later use.
#' @examples
#' # Loading and re-coding ANES 1980 data:
#' data(LC1980)
#' LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
#' self <- LC1980[, 1]
#' stimuli <- LC1980[, -1]
#'
#' # Prepare data for model fitting, using defaults:
#' dat <- prep_data(self, stimuli)
#'
#' # Prepare data for model fitting, using using alternative settings:
#' dat2 <- prep_data(self, stimuli, allow_miss = 0, req_unique = 3)
#'
#' # Obtain the data that are included in the analysis:
#' self2 <- self[dat2$keep]
#' stimuli2 <- stimuli[dat2$keep, ]

prep_data <- function(self = NULL, stimuli,
                      prefs = NULL,
                      allow_miss = 2,
                      req_valid = NA, req_unique = 2, B = NULL, group_id = NULL, quiet = FALSE) {

  #dimnames(stimuli) <- NULL
  #self <- as_numeric(self)

  N_orig <- nrow(stimuli)
  N_obs_orig <- sum(!is.na(as.numeric(as.matrix(stimuli))))

  if (is.null(self)) {
    V_supplied <- FALSE
    self <- round(rep(mean(range(unlist(stimuli), na.rm = TRUE)), nrow(stimuli)))
    message("Note: No self-placement data were supplied and respondent positions will therefore not be estimated.")
  } else {
    V_supplied <- TRUE
    if (length(self) != nrow(stimuli)) {
      stop("The length of the self-placement vector does not match the number of rows in the stimuli data.")
    }
  }

  stimulicols <- apply(stimuli, 2, function(x) sum(!is.na(x))) > 0
  stimuli <- stimuli[, stimulicols]

  if (any_true_after_false(stimulicols)) {
    warning("Some columns in the supplied stimuli data do not contain any non-NA values and will be ignored. This affects the indexing of the stimuli in the output because at least one column containing data appears after the empty column(s). You may want to drop empty columns to avoid confusion.")
    have_warned_on_missing_cols <- TRUE
  } else {
    have_warned_on_missing_cols <- FALSE
  }

  # req_valid takes precedence over allow_miss they are not consistent:
  if (!is.na(req_valid)) { allow_miss <- ncol(stimuli) - req_valid }

  all_dat <- cbind(stimuli, self)
  range_orig <- range(all_dat, na.rm = TRUE)

  # Counting number of unique stimuli positions reported per respondent:
  n_unique <- apply(stimuli, 1, function(x) {length(unique(x[!is.na(x)]))} )

  # Center position-data if necessary:
  if (range_orig[1] >= 0) {
    stimuli <- (stimuli - range_orig[1]) - ((range_orig[2] - range_orig[1]) / 2)
    self <- (self - range_orig[1]) - ((range_orig[2] - range_orig[1]) / 2)
  }

  range_new <- range(cbind(stimuli, self), na.rm = TRUE)

  if (!is.null(group_id)) {
    has_group_id <- !is.na(group_id) & is.finite(group_id)
    group_id_fac <- as.factor(group_id)
    group_id <- as.numeric(group_id_fac)
    if (!complete_sequence_integers(group_id[has_group_id])) {
      stop("Did not succeed in converting the supplied group_id to a suitable index format. You could try transforming it to integers covering all values from 1 to the total number of groups.")
    }
  } else {
    group_id_fac <- NULL
    group_id <- rep(1, nrow(stimuli))
    has_group_id <- rep(TRUE, N_orig)
  }

  if (!is.null(prefs)) {
    prefs <- prefs / max(prefs, na.rm = T)

    # Rescale preferences to range from 0 to 1 per for each individual:
    suppressWarnings(pref_min <- apply(prefs, 1, min, na.rm = T))
    suppressWarnings(pref_max <- apply(prefs, 1, max, na.rm = T))
    sel <- pref_min != pref_max
    prefs[sel,] <- (prefs[sel,] - pref_min[sel]) / (pref_max[sel] - pref_min[sel])

    # Indicators of non-constant preference data for each respondent:
    has_var <- apply(prefs, 1, var, na.rm = T) > 0

    # Which observations are missing either prefs or positions:
    is_any_na <- apply((is.na(prefs) | is.na(stimuli)), 2, as.numeric)

    keep <- !is.na(self) & apply(is_any_na, 1, sum) <= allow_miss & n_unique >= req_unique & has_var & has_group_id
    prefs <- prefs[keep,]
    U_supplied = TRUE
  } else {
    keep <- !is.na(self) & apply(is.na(stimuli), 1, sum) <= allow_miss & n_unique >= req_unique & has_group_id
    U_supplied = FALSE
  }

  if (sum(keep) == 0) {
    stop('The current selection criteria will drop all respondents from the analysis. You may want to consider increasing the argument "allow_miss" or instead specify a reasonably low value of "req_valid".')
  }

  stimuli <- stimuli[keep, ]
  self <- self[keep]
  group_id <- group_id[keep]
  if (!is.null(group_id_fac)) {
    group_id_fac <- group_id_fac[keep]
  }

  stimulicols2 <- apply(stimuli, 2, function(x) sum(!is.na(x))) > 0
  stimuli <- stimuli[, stimulicols2]

  if (any_true_after_false(stimulicols2) & !have_warned_on_missing_cols) {
    warning(paste0("Some columns in the supplied stimuli data do not contain any non-NA values after inclusion criteria have been applied. This affects the indexing of the stimuli in the output because at least one column containing data appears after the empty column(s). Using more liberal inclusion criteria (e.g. a higher allow_miss argument) may avoid this issue. Alternatively, you may want to remove the following stimuli from the data to avoid confusion: ", paste(which(!stimulicols2), collapse = ", "), "."))
  }

  mean_spos <- apply(stimuli, 2, mean, na.rm = T)

  if (sum(stimuli < 0, na.rm = T) > 0) {
    L <- which.max(apply(stimuli < 0, 2, sum, na.rm = T))
  } else {
    L <- which.min(mean_spos)
  }

  if (sum(stimuli > 0, na.rm = T) > 0) {
    R <- which.max(apply(stimuli > 0, 2, sum, na.rm = T))
  } else {
    R <- which.max(mean_spos)
  }

  # Coding Y as a long-form sparse matrix:
  stimuli_vec <- as.numeric(as.matrix(stimuli))
  if (!is.null(prefs)) {
    prefs_vec <- as.numeric(as.matrix(prefs))
    drop <- is.na(stimuli_vec) | is.na(prefs_vec)
    prefs_vec <- prefs_vec[!drop]
  } else{
    drop <- is.na(stimuli_vec)
    prefs_vec <- rep(0, sum(!drop))
  }
  ii <- rep(1:nrow(stimuli), ncol(stimuli))[!drop]
  jj <- rep(1:ncol(stimuli), each = nrow(stimuli))[!drop]
  stimuli_vec <- stimuli_vec[!drop]

  if(is.null(B)) { B <- max(abs(c(stimuli_vec, self)), na.rm = T) }

  datlist <- list(J = ncol(stimuli), N = nrow(stimuli), B = B, B2 = ceiling(B), N_obs = length(stimuli_vec),
       V = self, Vi = floor(self) + ceiling(B) + 1, Y = stimuli_vec, U = prefs_vec, L = L, R = R, K = range_orig[2] - range_orig[1] + 1,
       ii = ii, jj = jj, gg = group_id, G = length(unique(group_id)), ggfac = group_id_fac, mean_spos = mean_spos, keep = keep, names = colnames(stimuli),
       CV = 0, holdout = rep(0, length(stimuli_vec)), V_supplied = V_supplied, U_supplied = U_supplied, N_orig = N_orig)
  class(datlist) <- c("list", "hbam_data")

  if (!quiet) {
    writeLines(paste0(
"Summary of prepared data (values for supplied data in parentheses)
- Number of respondents: ", nrow(stimuli), " (", N_orig, ")
- Number of stimuli: ", ncol(stimuli), " (", length(stimulicols), ")
- Number of stimuli obs.: ", length(stimuli_vec), " (", N_obs_orig, ")
- Range of observations: [", range_new[1], ", ", range_new[2], "] ([", range_orig[1], ", ", range_orig[2], "])"
    ))
  }

  return(datlist)
}

all_integers <- function(vec) {
  all(is.numeric(vec) & !is.na(vec)) && all(vec == as.integer(vec))
}

complete_sequence_integers <- function(vec) {
  unique_vals <- unique(vec)

  if (!all_integers(vec)) {
    return(FALSE)  # If not all integers, return FALSE
  }

  sorted_unique <- sort(unique_vals)
  expected_sequence <- seq(min(sorted_unique), max(sorted_unique))

  is_complete_sequence <- length(sorted_unique) == length(expected_sequence) &&
    all(sorted_unique == expected_sequence)

  is_complete_sequence
}

any_true_after_false <- function(x) {
  if (sum(!x) > 0) {
    first_false_index <- which(!x)[1]
    if (first_false_index < length(x)) {
      any(x[(first_false_index + 1):length(x)])
    } else {
      FALSE
    }
  } else {
    FALSE
  }
}

#' 1980 Liberal-Conservative Scales
#'
#' Liberal-Conservative 7-point scales from the 1980 National Election Study.
#' Includes (in order) self-placement, and rankings of Carter, Reagan, Kennedy,
#' Anderson, Republican party, Democratic Party. Stored as a matrix of integers.
#' The numbers 0, 8, and 9 are considered to be missing values.
#'
#' @docType data
#'
#' @usage data(LC1980)
#'
#' @keywords datasets
#'
#' @source American National Election Studies.
#'
#' This dataset was originally part of the `basicspace` package under the same name ("LC1980").
#'
#' @examples
#' data(LC1980)
#' LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
#' head(LC1980)
"LC1980"

#' Fit an HBAM model
#'
#' Fit a Hierarchical Bayesian Aldrich-McKelvey model using automatically tuned Hamiltonian Monte Carlo sampling (NUTS) via `rstan`.
#'
#' @export
#' @param self An optional numerical vector of N ideological self-placements. Any missing data must be coded as NA. If this argument is not supplied (either here or in a previous call to `prep_data()`), respondent positions will not be estimated. If the data have been prepared in advance via the `prep_data()` function, the argument supplied here will be ignored.
#' @param stimuli An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param model Character: Name of the model to be used. Defaults to HBAM. The available models are described under Details.
#' @param allow_miss Integer specifying how many missing stimulus positions should be accepted for an individual still to be included in the analysis. This argument will not be used if the data have been prepared in advance via the `prep_data()` function. Defaults to 2.
#' @param req_valid Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is `req_valid = J - allow_miss`, but if specified, `req_valid` takes precedence. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param req_unique Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is `req_unique = 2`. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param data List of data that have been prepared in advance via the `prep_data()` function. Not required if the arguments `self` and `stimuli` are provided.
#' @param prefs An N × J matrix of numerical stimulus ratings or preference scores. These data are only required by the HBAM_R_MINI model and will be ignored when fitting other models.
#' @param group_id Vector of length N identifying which group each respondent belongs to. The format can be factor, character, integer, or numeric. Respondents with NAs on `group_id` will be dropped when `group_id` is supplied. These data are only required by models with `"MULTI"` in their name and will be ignored when fitting other models.
#' @param pars A vector of character strings specifying parameters of interest. If `include = TRUE`, only samples for parameters named in pars are stored in the fitted results. Conversely, if `include = FALSE`, samples for all parameters except those named in pars are stored in the fitted results. The default is to store results for "alpha", "beta", "chi", "lambda", and "theta".
#' @param extra_pars A vector of character strings specifying parameters or generated quantities to be added to `pars`. This makes it easy to add one or several additional parameters of interest, without having to repeat the default `pars` vector. The default is `NULL`.
#' @param include Logical scalar defaulting to `TRUE` indicating whether to include or exclude the parameters given by the pars argument. If `FALSE`, only entire multidimensional parameters can be excluded, rather than particular elements of them.
#' @param chains A positive integer specifying the number of Markov chains. Defaults to 4.
#' @param cores The number of cores to use when executing the Markov chains in parallel. By default, all detected physical cores will be used if `chains` is equal to or higher than the number of cores.
#' @param warmup A positive integer specifying the number of warmup (aka burn-in) iterations per chain. If step-size adaptation is on (which it is by default), this also controls the number of iterations for which adaptation is run (and hence these warmup samples should not be used for inference). The number of warmup iterations should be smaller than `iter`.
#' @param iter A positive integer specifying the number of iterations for each chain (including warmup).
#' @param seed A positive integer specifying an optional seed for reproducibility. If this argument is not supplied, a random seed will be generated and the function will produce slightly different results on each run.
#' @param control A named list of parameters to control the sampler's behavior. See the documentation for `rstan::stan` for more details.
#' @param sigma_alpha A positive numeric value specifying the standard deviation of the prior on the shift parameters in the FBAM model, or the standard deviation of the parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to B / 5, where B measures the length of the survey scale as the number of possible placements on one side of the center.
#' @param sigma_beta A positive numeric value specifying the standard deviation of the prior on the logged stretch parameters in the FBAM model, or the standard deviation of the logged parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to .3.
#' @param sigma_mu_alpha A positive numeric value specifying the standard deviation of the prior on the group-means of the shift parameters in MULTI-type models. Defaults to B / 10.
#' @param sigma_mu_beta A positive numeric value specifying the standard deviation of the prior on the group-means of the logged stretch parameters in MULTI-type models. Defaults to .2.
#' @param ... Arguments passed to `rstan::sampling()`.
#' @details This package provides several alternative models that can be selected using the names below. Users who are unsure which model to use are advised to use the default HBAM model. If speed or sampling diagnostics are an issue, HBAM_MINI may provide a useful alternative.
#'
#' **HBAM** is the default model, which allows for scale flipping and employs hierarchical priors on the shift and stretch parameters. It also models heteroskedastic errors that vary by both individual and stimuli. Compared to the model in Bølstad (2024), this version has been slightly revised to provide faster sampling. A key difference from the original model is that the respondent positions are not treated as parameters, but rather calculated as a function of self-placements, individual-level parameters, and simulated errors. This makes the model considerably faster, while yielding very similar results. The model simulates errors in the self-placements of the same magnitude as that with which the respondent in question places the stimulus with the smallest errors. All models in the package use this approach.
#'
#' **HBAM_MULTI** is a version that models differences between groups defined by the user. It requires a vector identifying the groups to be supplied as the argument `group_id`. The model gives each group separate hyperparameters for the locations of the prior distributions for the shift and stretch parameters. Rather than shrinking the estimates toward the mode for the whole dataset, this model shrinks the estimates toward the mode for the group. The vectors of hyperparameters are called `mu_alpha` and `mu_beta` and are constructed to have means of 0. The scales of the priors on these hyperparameters can be set by the user via the arguments `sigma_mu_alpha` and `sigma_mu_beta`. The default values are B / 10 and .2, respectively. (Here, B measures the length of the survey scale as the number of possible placements on one side of the center.) One potential use for this model is to supply self-placements as `group_id`, and thus give each self-placement group its own prior distribution for the shift and stretch parameters.
#'
#' **HBAM_NF** (formerly HBAM_0) is a version of the HBAM model that does not allow for scale flipping. This may be useful if there are truly zero cases of scale flipping in the data. Such scenarios can be created artificially, but may also arise in real data. For example, expert surveys appear unlikely to contain many instances of scale flipping. For data that contain zero cases of flipping, models that allow for flipping contain superfluous parameters that lead to inefficient sampling. Models that do not allow for flipping will sample faster and typically yield slightly more accurate estimates. Such models are therefore usually preferable when no flipping is present.
#'
#' **HBAM_MULTI_NF** is a version of the HBAM_MULTI model that does not allow for scale flipping.
#'
#' **HBAM_MINI** is a version of the HBAM model that assumes the prediction errors in the stimuli placements to be homoskedastic. This model tends to sample faster than the standard HBAM model while yielding very similar point estimates. For large datasets, this model may provide a reasonable compromise between model complexity and estimation speed.
#'
#' **FBAM** is a version of the HBAM model with fixed hyperparameters to allow fitting via optimization rather than MCMC -- which can be useful for large data sets. This model allows the user to specify the scales of the priors for the shift and (logged) stretch parameters via the arguments `sigma_alpha` and `sigma_beta`. The default values are B / 5 and .3, respectively. These defaults are intended to be realistic and moderately informative. Users who want to control the degree of shrinkage of the individual-level parameters may find it useful to fit this model -- or other FBAM models -- via either MCMC or optimization.
#'
#' **FBAM_MULTI** is a version of the FBAM model that shares the group-modeling features of the HBAM_MULTI model. It allows the user to set the scales of the priors for the shift and stretch parameters via the arguments `sigma_alpha` and `sigma_beta`, and set the scales of the priors on `mu_alpha` and `mu_beta` via the arguments `sigma_mu_alpha` and `sigma_mu_beta`.
#'
#' **FBAM_MULTI_NF** is a version of the FBAM_MULTI model that does not allow for scale flipping.
#'
#' **HBAM_R_MINI** is a version of the HBAM_MINI model that incorporates the rationalization component of the ISR model by Bølstad (2020). This model requires additional data to be supplied as the argument `prefs`: An N × J matrix of stimuli ratings from the respondents. The rationalization part of the model is simplified relative to the original ISR model: The direction in which respondents move disfavored stimuli is estimated as a common expectation for each possible self-placement on the scale.
#'
#' **BAM** is an unpooled model with wide uniform priors on the shift and stretch parameters. It is similar to the JAGS version introduced by Hare et al. (2015), although the version included here has been adjusted to yield stretch parameters with an average of approximately one (and thus produce a scale similar to those of the other models in the package). Like the other models, this version of the BAM model also simulates errors in the self-placements to yield a realistic level of uncertainty. While this model is simple and fast, it tends to overfit the data and produce invalid posterior distributions for some respondent positions (see Bølstad 2024). However, it could potentially be useful as a baseline for model comparisons in situations where respondent positions are not of interest.
#'
#' **HBAM_2** has been replaced by the more general HBAM_MULTI model.
#'
#' These models can also be used in situations where self-placements are not available and the only goal is to estimate stimulus positions or respondents' shift and stretch parameters. While the latent respondent positions will not be estimated, all other parameters are unaffected if the argument `self` is dropped when calling `prep_data()` or `hbam()`.
#'
#' See the `hbamr` vignette for a table summarizing the key characteristics of the available models.
#'
#' @return An object of S4 class `stanfit`.

#' @references
#' - Bølstad, Jørgen (2024). Hierarchical Bayesian Aldrich-McKelvey Scaling. \emph{Political Analysis}. 32(1): 50–64. \doi{10.1017/pan.2023.18}.
#' - Bølstad, Jørgen (2020). Capturing Rationalization Bias and Differential Item Functioning: A Unified Bayesian Scaling Approach. \emph{Political Analysis} 28(3): 340–355.
#' - Hare, Christopher et al. (2015). Using Bayesian Aldrich-McKelvey Scaling to Study Citizens' Ideological Preferences and Perceptions. \emph{American Journal of Political Science} 59(3): 759–774.
#'
#' @examples
#' \donttest{
#' # Loading and re-coding ANES 1980 data:
#' data(LC1980)
#' LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
#'
#' # Making a small subset of the data for illustration:
#' self <- LC1980[1:100, 1]
#' stimuli <- LC1980[1:100, -1]
#'
#' # Fitting the HBAM_MINI model, obtaining 1000 draws:
#' fit_hbam_mini <- hbam(self, stimuli, model = "HBAM_MINI",
#'                    warmup = 500, iter = 1000, chains = 2)
#'
#' # Preparing the data before fitting, requiring complete responses:
#' dat <- prep_data(self, stimuli, allow_miss = 0)
#' fit_hbam_mini <- hbam(data = dat, model = "HBAM_MINI",
#'                    warmup = 500, iter = 1000, chains = 2)
#'
#' # Obtaining posterior summaries for the latent stimulus positions:
#' theta_est <- get_est(fit_hbam_mini, par = "theta")
#'
#' # Obtaining posterior summaries for the latent respondent positions
#'   # in a format matching the rows in the original dataset:
#' chi_est <- get_est(fit_hbam_mini, par = "chi", format_orig = TRUE)
#'
#' # Fitting the FBAM_MULTI_NF model with self-placements as group_id:
#' fit_fbam_multi_nf <- hbam(self, stimuli, group_id = self, model = "FBAM_MULTI_NF",
#'                        warmup = 500, iter = 1000, chains = 2)
#' }

hbam <- function(self = NULL, stimuli = NULL, model = "HBAM", allow_miss = 2, req_valid = NA,
                 req_unique = 2, prefs = NULL, group_id = NULL, data = NULL,
                 pars = c("alpha", "beta", "chi", "lambda", "theta"),
                 extra_pars = NULL, include = TRUE,
                 chains = 4, cores = parallel::detectCores(logical = FALSE),
                 warmup = 1000, iter = 2000,
                 seed = sample.int(.Machine$integer.max, 1),
                 control = list(max_treedepth = 7),
                 sigma_alpha = NULL, sigma_beta = .3,
                 sigma_mu_alpha = NULL, sigma_mu_beta = .2, ...) {
  if (!model %in% c("HBAM", "HBAM_NF", "HBAM_MINI", "HBAM_MULTI", "HBAM_MULTI_NF",
                    "HBAM_R_MINI", "BAM", "FBAM", "FBAM_MULTI", "FBAM_MULTI_NF")) { stop(paste(model, "is not a valid model choice.")) }
  if (!is.null(data) & (!is.null(self) | !is.null(stimuli))) { message("Note: When pre-prepared data are supplied, other data arguments will be ignored.") }
  if (is.null(data) & (is.null(self) | is.null(stimuli))) { stop("Required data not supplied.") }
  if (is.null(data)) {
    dat <- hbamr::prep_data(self, stimuli, prefs, allow_miss = allow_miss, req_valid = req_valid, req_unique = req_unique, group_id = group_id)
    writeLines("")
  } else {
    if (inherits(data, "hbam_data")) {
      dat <- data
    } else {
      stop("Supplied data is not of hbam_data class. Please use the prep_data() function from the hbamr package to prepare the data.")
    }
  }
  if (hasArg(prep_data)) { message("Note: The prep_data argument has been deprecated - please remove it. If a data argument is supplied, the data will not be further prepared before fitting (and vice versa).") }
  if (grepl("MULTI", model) & dat$G <= 1) { stop("No group_id supplied for MULTI-type model.") }
  if (!grepl("MULTI", model) & dat$G > 1) { message("Note: The supplied group_id will not be used as the chosen model is not a MULTI-type model.") }
  if (model == "BAM" | grepl("_NF", model)) { pars = c("alpha", "beta", "chi", "theta") }
  if (!is.null(extra_pars)) { pars <- c(pars, extra_pars) }
  if (!dat$V_supplied) {
    # pars <- pars[pars != "chi"]
    if (model == "HBAM_R_MINI") {
      stop("The HBAM_R_MINI model requires self-placement data.")
    }
  }
  if (model == "HBAM_R_MINI" & !dat$U_supplied) {
    stop("The HBAM_R_MINI model requires preference data.")
  }
  if (model == "HBAM_R_MINI" & !("gamma" %in% pars)) {
    pars <- c(pars, "gamma")
  }

  if (is.null(sigma_alpha)) { sigma_alpha <- dat$B / 5 }
  if (is.null(sigma_mu_alpha)) { sigma_mu_alpha <- dat$B / 10 }
  dat$sigma_alpha_fixed <- sigma_alpha
  dat$sigma_mu_alpha <- sigma_mu_alpha
  dat$sigma_beta_fixed <- sigma_beta
  dat$sigma_mu_beta <- sigma_mu_beta
  dat$MCMC <- 1

  dat <- add_model_features(dat, model)

  set.seed(seed)
  init_ll <- lapply(1:chains, function(id) inits_omni(id, dat))

  stanmodels[[1]]@model_name <- model

  arglist <- list(...)
  arglist$prep_data <- NULL
  arglist$object <- stanmodels[[1]]
  arglist$data = dat
  arglist$init = init_ll
  arglist$chains = chains
  arglist$cores = cores
  arglist$warmup = warmup
  arglist$iter = iter
  arglist$seed = seed
  arglist$control = control
  arglist$pars = pars
  arglist$include = include

  out <- do.call(rstan::sampling, arglist)
  out@.MISC$hbam_data <- dat
  return(out)
}

add_model_features <- function(data, model) {
  if (grepl("_MINI", model)) {
    data$het <- 0
  } else {
    data$het <- 1
  }
  if (grepl("_NF", model)) {
    data$flip <- 0
  } else {
    data$flip <- 1
  }
  if (grepl("FBAM", model)) {
    data$fixed <- 1
  } else {
    data$fixed <- 0
  }
  if (grepl("MULTI", model)) {
    data$group <- 1
  } else {
    data$group <- 0
  }
  if (model == "BAM") {
    data$bam <- 1
    data$flip <- 0
  } else {
    data$bam <- 0
  }
  if (model == "HBAM_R_MINI") {
    data$rat <- 1
  } else {
    data$rat <- 0
  }
  return(data)
}

#' Plot individual parameter estimates over self-placements
#'
#' Create a boxplot of individual parameter point estimates from an HBAM model over self-placements
#'
#' @export
#' @param object An object of class `stanfit` produced by `hbam()`, a list produced by `fbam()`, or a `list` of such objects, which will produce a faceted plot.
#' @param par Character: Name of the parameter to be plotted. One of the following: `"alpha"`, `"beta"`, `"abs_beta"`, `"lambda"`, or `"chi"`. Defaults to `"chi"`. Further individual-level parameters like `"eta"` can be specified if these have been passed to `hbam()` via the argument `extra_pars` when fitting the model. (Note that homoskedastic models have no `"eta"` parameters and "NF"-type models have no `"lambda"` or `"kappa"` parameters.)
#' @param estimate Character: Specifying which type of posterior point estimate to use. One of `"median"` and `"mean"`. Defaults to `"median"`. This only applies for `stanfit` objects.
#' @param names An optional character vector of model names of same length as the supplied list of models.
#' @param parlabel An optional character containing an alternative label for the parameter (will be parsed if passed as an expression).
#' @param fill Fill color of boxes. Passed on to `ggplot2::geom_boxplot()`.
#' @param color Color of outer lines. Passed on to `ggplot2::geom_boxplot()`.
#' @param width Width of boxes. Passed on to `ggplot2::geom_boxplot()`.
#' @param alpha Number in \[0,1\]: Inverse level of transparency for fill color.
#' @param outlier.size Size of dots representing outliers. Passed on to `ggplot2::geom_boxplot()`.
#' @param median_color Color of solid line representing the median.
#' @param median_lwd Thickness of solid line representing the median.
#' @return A `ggplot` object.
#'

plot_over_self <- function(object, par = "chi", estimate = "median", names = NULL, parlabel = NULL,
                           fill = "#2166AC", color = "#053061", width = .7, alpha = .5, outlier.size = 0.3,
                           median_color = "black", median_lwd = .7) {
  if(is.null(parlabel)) { parlabel <- par}
  if((length(object) == 1 & inherits(object, "stanfit")) | is_fbam(object)) {
    pd <- get_pd(object, par, estimate)
    p <- ggplot2::ggplot(pd, ggplot2::aes(.data$V, .data$parameter)) + ggplot2::geom_boxplot(fill = fill, color = color, width = width, alpha = alpha, outlier.size = outlier.size) +
      ggplot2::xlab("Self-placement") + ggplot2::ylab(par)
    md <- ggplot2::ggplot_build(p)$data[[1]]
    p <- p + ggplot2::geom_segment(data = md, aes(x = .data$xmin, xend = .data$xmax,
                                   y = .data$middle, yend = .data$middle), colour = median_color, linewidth = median_lwd)
  } else {
    pd <- vector()
    for (m in 1:length(object)) {
      if (is.null(names)) {
        if (inherits(object[[m]], "stanfit")) {
          name <- object[[m]]@model_name
        } else {
          name <- paste0("Model ", m)
        }
      } else { name <- names[m] }
      pd_temp <- get_pd(object[[m]], par, estimate)
      pd <- rbind(pd, dplyr::bind_cols(pd_temp, model = rep(name, nrow(pd_temp))))
    }
      if (is.null(names)) {
        pd$model <- factor(pd$model, levels = unique(pd$model), labels = unique(pd$model)) } else {
        pd$model <- factor(pd$model, levels = names, labels = names) }
      p <- ggplot2::ggplot(pd, ggplot2::aes(.data$V, .data$parameter)) + ggplot2::geom_boxplot(fill = fill, color = color, width = width, alpha = alpha, outlier.size = outlier.size) +
        ggplot2::xlab("Self-placement") + ggplot2::ylab(par) +
        ggplot2::facet_wrap(~model, scales = "free") + ggplot2::ylab(parlabel)
      md <- ggplot2::ggplot_build(p)$data[[1]]
      md$model <- factor(md$PANEL, levels = as.numeric(unique(pd$model)), labels = levels(pd$model))
      p <- p + ggplot2::geom_segment(data = md, aes(x = .data$xmin, xend = .data$xmax,
                                            y = .data$middle, yend = .data$middle), colour = median_color, linewidth = median_lwd)
  }
  return(p)
}

get_pd <- function(object, par, estimate){
  if (inherits(object, "stanfit")) {
    data <- object@.MISC$hbam_data
    if (is.null(data) || !inherits(data, "hbam_data")) {
      stop("Could not find the data used for fitting within the supplied object. You need to supply an object produced by hbam() or fbam().") }
    if (par == "abs_beta") {
      draws <- as.matrix(rstan::extract(object, pars = "beta")$beta)
      param <- data.frame(
        median = apply(draws, 2, function(x) median(abs(x)) ),
        mean = apply(draws, 2, function(x) mean(abs(x)) ))
      names(param) <- c("50%", "mean")
    } else {
      param <- get_est(object, par)
      if (par == "eta") {
        param[, 1] <- sqrt(param[, 1]) / data$J # To get average sigma for each i over j.
        param[, 3] <- sqrt(param[, 3]) / data$J
        par <- "sqrt(eta) / J"
      }
    }
    if (estimate == "median") { pd <- dplyr::bind_cols(parameter = param$`50%`, V = as.ordered(data$V)) }
    if (estimate == "mean") { pd <- dplyr::bind_cols(parameter = param$mean, V = as.ordered(data$V)) }
  } else {
    if (inherits(object, "list")) {
      data <- object$hbam_data
      if (is.null(data) || !inherits(data, "hbam_data")) {
        stop("Could not find the data used for fitting within the supplied object. You need to supply an object produced by hbam() or fbam().") }
      if (par == "abs_beta") {
        param <- get_est(object, "beta")
        param <- abs(param)
      } else {
        param <- get_est(object, par)
      }
      pd <- data.frame(parameter = param[, 1], V = as.ordered(data$V))
      names(pd)[1] <- "parameter"
    }
  }
  return(pd)
}

is_fbam <- function(object) {
  output <- FALSE
  if (!is.null(names(object))) {
    if (names(object)[1] == "par" & names(object)[3] == "return_code" & inherits(object, "list")) {
      output <- TRUE
    }
  }
  return(output)
}

#' Extract data for plotting results from an HBAM model
#'
#' Extract data for plotting results from an HBAM model.
#'
#' @export
#' @param object An instance of class `stanfit` produced by `hbam()` or a list produced by `fbam()`.
#' @param n_draws Integer specifying the number of posterior draws to use when illustrating the uncertainty of the population distribution. This only applies for `stanfit` objects.
#' @param seed A positive integer specifying an optional seed for reproducibility. The seed is used to select respondent position draws for illustrating uncertainty. This only applies for `stanfit` objects.
#' @return A list of three `tibble`s: The first element contains the posterior mean stimulus positions, as well as the x- and y-values of the posterior modes (which can be useful for labeling the distributions). The second element contains the posterior draws for the stimulus positions (which can be used to calculate marginal posterior densities). The third element contains the selected number of posterior draws for each respondent (which form the key ingredient for `plot_respondents`).
#'

get_plot_data <- function(object, n_draws = 15, seed = 1) {
  if (inherits(object, "stanfit")) {
    set.seed(seed)
    chi_draws <- as.data.frame((rstan::extract(object, pars = "chi")$chi))
    chi_draws <- chi_draws[sample(nrow(chi_draws), n_draws), ]
    chi_draws$draw_no <- rownames(chi_draws)
    chi_draws <- pivot_longer(chi_draws, starts_with("V"), values_to = "chi")
    s_draws <- as.data.frame(rstan::extract(object, pars = "theta")$theta)

    # Use stimuli-names if available:
    if (!is.null(names(attr(object@sim$samples[[1]], "args")$init_list$theta_raw))) {
      colnames(s_draws) <- names(attr(object@sim$samples[[1]], "args")$init_list$theta_raw) }
    s_draws <- pivot_longer(s_draws, everything())
    s_draws$name <- factor(s_draws$name)

    label <- s_draws %>%
      dplyr::group_by(.data$name) %>%
      dplyr::summarize(x = mean(.data$value), mode_x = get_post_mode_x(.data$value), mode_y = get_post_mode_y(.data$value))
    # label$y_adj <- label$y + mean(label$y) * .07
  } else {
    if (inherits(object, "list")) {
      chi_draws <- dplyr::as_tibble(data.frame(chi = object$par[grepl( "chi[", names(object$par), fixed = TRUE)]))
      chi_draws$draw_no <- 1
      s_draws <- data.frame(theta = object$par[grepl( "theta[", names(object$par), fixed = TRUE)])
      label <- dplyr::as_tibble(data.frame(x = s_draws$theta, mode_x = s_draws$theta, mode_y = NA))
      label$name <- rownames(s_draws)
      s_draws <- dplyr::as_tibble(s_draws)
    }
  }
  return(list(s_label = label, s_draws = s_draws, chi_draws = chi_draws))
}

#' Perform K-fold cross-validation
#'
#' This function performs K-fold cross-validation for an HBAM or FBAM model in order to estimate the expected log pointwise predictive density for a new dataset (ELPD). Multiple chains for one or more folds can be run in parallel using the `future` package.
#'
#' @export
#' @param self A numerical vector of N ideological self-placements. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param stimuli An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param model Character: Name of the model to be used. Defaults to HBAM.
#' @param allow_miss Integer specifying how many missing stimulus positions should be accepted for an individual still to be included in the analysis. This argument will not be used if the data have been prepared in advance via the `prep_data()` function. Defaults to 0.
#' @param req_valid Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is `req_valid = J - allow_miss`, but if specified, `req_valid` takes precedence. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param req_unique Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is `req_unique = 2`. This argument will not be used if the data have been prepared in advance via the `prep_data()` function.
#' @param prep_data Logical: Should the data be prepared before fitting the model? (Or have the data been prepared in advance by first running the `prep_data()` and `prep_data_cv()` functions)? If so, set `prep_data = FALSE`.) Defaults to `prep_data = TRUE`.
#' @param data A list of data produced by `prep_data()` followed by `prep_data_cv()`.
#' @param prefs An N × J matrix of numerical stimulus ratings or preference scores. These data are only required by the HBAM_R_MINI model and will be ignored when fitting other models.
#' @param group_id Integer vector of length N identifying which group each respondent belongs to. The supplied vector should range from 1 to the total number of groups in the data, and all integers between these numbers should be represented in the supplied data. These data are only required by models with "MULTI" in their name and will be ignored when fitting other models.
#' @param K An integer above 2, specifying the number of folds to use in the analysis. Defaults to 10.
#' @param chains A positive integer specifying the number of Markov chains to use per fold. Defaults to 2.
#' @param warmup A positive integer specifying the number of warmup (aka burn-in) iterations per chain. It defaults to 1000. The number of warmup iterations should be smaller than `iter`.
#' @param iter A positive integer specifying the number of iterations for each chain (including warmup). It defaults to 3000 as running fewer chains for longer is a more efficient way to obtain a certain number of draws (and cross-validation can be computationally expensive).
#' @param seed An integer passed on to `set.seed` before creating the folds to increase reproducibility and comparability. Defaults to 1 and only applies to fold-creation when the argument `prep_data` is `TRUE`. The supplied `seed` argument is also used to generate seeds for the sampling algorithm.
#' @param control A named list of parameters to control the sampler's behavior. See the documentation for `rstan::stan` for more details.
#' @param sigma_alpha A positive numeric value specifying the standard deviation of the prior on the shift parameters in the FBAM model, or the standard deviation of the parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to B / 4, where B measures the length of the survey scale as the number of possible placements on one side of the center.
#' @param sigma_beta A positive numeric value specifying the standard deviation of the prior on the logged stretch parameters in the FBAM model, or the standard deviation of the logged parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to .35.
#' @param sigma_mu_alpha A positive numeric value specifying the standard deviation of the prior on the group-means of the shift parameters in MULTI-type models. Defaults to B / 5.
#' @param sigma_mu_beta A positive numeric value specifying the standard deviation of the prior on the group-means of the logged stretch parameters in MULTI-type models. Defaults to .3.
#' @param ... Arguments passed to `rstan::sampling()`.
#' @return A list of classes `kfold` and `loo`, which contains the following named elements:
#'    * `"estimates"`: A `1x2` matrix containing the ELPD estimate and its standard error. The columns have names `"Estimate"` and `"SE"`.
#'    * `"pointwise"`: A `Nx1` matrix with column name `"elpd_kfold"` containing the pointwise contributions for each data point.
#' @examples
#' \donttest{
#' # Loading and re-coding ANES 1980 data:
#' data(LC1980)
#' LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
#'
#' # Making a small subset of the data for illustration:
#' self <- LC1980[1:50, 1]
#' stimuli <- LC1980[1:50, -1]
#'
#' # Preparing to run chains in parallel using 2 cores via the future package:
#'   # Note: You would normally want to use all physical cores for this.
#' future::plan(future::multisession, workers = 2)
#'
#' # Performing 4-fold cross-validation for the HBAM_MINI model:
#'   # Note: You would typically want to run the chains for more iterations.
#' cv_hbam_mini <- hbam_cv(self, stimuli, model = "HBAM_MINI", K = 4,
#'                         chains = 1, warmup = 500, iter = 1000)
#'
#' # Performing 4-fold cross-validation for the FBAM model:
#' cv_fbam <- hbam_cv(self, stimuli, model = "FBAM", K = 4,
#'                         chains = 1, warmup = 500, iter = 1000)
#'
#' # Comparing the results using the loo package:
#' loo::loo_compare(list(HBAM_MINI = cv_hbam_mini,
#'                  FBAM = cv_fbam))
#'
#' # Stop the cluster of parallel sessions:
#' future::plan(future::sequential)
#' }

hbam_cv <- function(self = NULL, stimuli = NULL, model = "HBAM",
                    allow_miss = 0, req_valid = NA, req_unique = 2,
                    prefs = NULL, group_id = NULL, prep_data = TRUE, data = NULL, K = 10,
                    chains = 2,
                    warmup = 1000, iter = 3000,
                    seed = 1,
                    control = list(max_treedepth = 7),
                    sigma_alpha = NULL, sigma_beta = .35,
                    sigma_mu_alpha = NULL, sigma_mu_beta = .3, ...){

  logColMeansExp <- function(x) {
    S <- nrow(x)
    matrixStats::colLogSumExps(x) - log(S) # This is an alternative way of calculating the col means to taking sum of exps and dividing by N before then logging
  }

  if (prep_data == TRUE) {
    dat <- hbamr::prep_data(self = self, stimuli = stimuli, prefs = prefs, allow_miss = allow_miss, req_valid = req_valid, req_unique = req_unique, group_id = group_id)
    dat_l <- hbamr::prep_data_cv(data = dat, K = K, seed = seed)
  } else {
    dat_l <- data
  }

  n_fold <- length(dat_l)

  if (is.null(sigma_alpha)) { sigma_alpha <- dat_l[[1]]$B / 4.0 }
  if (is.null(sigma_mu_alpha)) { sigma_mu_alpha <- dat_l[[1]]$B / 5.0 }
  for (k in 1:n_fold) {
    dat_l[[k]]$sigma_alpha_fixed <- sigma_alpha
    dat_l[[k]]$sigma_mu_alpha <- sigma_mu_alpha
    dat_l[[k]]$sigma_beta_fixed <- sigma_beta
    dat_l[[k]]$sigma_mu_beta <- sigma_mu_beta
    dat_l[[k]] <- add_model_features(dat_l[[k]], model)
  }

  RNGkind("L'Ecuyer-CMRG")
  set.seed(seed)

  # Function to run all chains and store only the log-likelihoods for held-out data to save memory:
  get_ll_list <- function() {
    p <- progressr::progressor(n_fold * chains)
    future.apply::future_lapply(1:(n_fold * chains),
                                function(i){
                                  k <- ceiling(i / chains) # Fold number
                                  init_l <- list(inits_omni(chain_id = i, dat = dat_l[[k]]))
                                  # Obtain chain:
                                  s <- rstan::sampling(stanmodels[[1]], data = dat_l[[k]], init = init_l, pars = "log_lik",
                                                       chains = 1, cores = 1, warmup = warmup, iter = iter, chain_id = i, seed = seed + i, control = control, refresh = 0, ...)
                                  # Calculate expected value of log-likelihood for each held-out observation:
                                  log_lik <- loo::extract_log_lik(s)
                                  draws <- dim(log_lik)[1]
                                  N_obs <- dim(log_lik)[2]
                                  heldout <- matrix(rep(dat_l[[k]]$holdout, each = draws), nrow = draws)
                                  log_lik_heldout <- log_lik * NA
                                  log_lik_heldout[heldout == 1] <- log_lik[heldout == 1]
                                  pointwise <-  matrix(logColMeansExp(log_lik_heldout), ncol= 1)

                                  p()

                                  return(pointwise)
                                },
                                future.seed = .Random.seed)

  }

  # Run all chains and show progress bar:
  pointwise_ll_list <- progressr::with_progress(get_ll_list())

  # Combine log-likelihoods from different chains by averaging, then combine values from different folds in a common vector:
  pointwise_ll <- vector()
  for(k in 1:n_fold){
    inchains <- (chains * k - (chains - 1)):(chains * k)
    pointwise_ll_mat <- t(matrix(unlist(pointwise_ll_list[inchains]), ncol = chains))
    pointwise_ll[dat_l[[k]]$holdout == 1] <- logColMeansExp(pointwise_ll_mat)[dat_l[[k]]$holdout == 1]
  }

  #out <- data.frame(
  #  ELPD = round(sum(pointwise_ll, na.rm = T), digits = 1),
  #  SE = round(as.numeric(sqrt(length(pointwise_ll) * var(pointwise_ll))), digits = 1))
  #rownames(out) <- model

  estimates = matrix(
    c(round(sum(pointwise_ll, na.rm = T), digits = 1),
      round(as.numeric(sqrt(length(pointwise_ll) * var(pointwise_ll))), digits = 1)),
    nrow = 1)
  rownames(estimates) <- "elpd_kfold"
  colnames(estimates) <- c("Estimate", "SE")

  pointwise_ll <- matrix(as.numeric(pointwise_ll), nrow = length(as.numeric(pointwise_ll)), ncol = 1)
  colnames(pointwise_ll) <- "elpd_kfold"

  out <- list("estimates" = estimates,
              "pointwise" = pointwise_ll)

  class(out) <- c("kfold", "loo")

  return(out)
}

#' Plot estimated respondent positions
#'
#' Plot the distribution of estimated respondent positions from an HBAM model.
#'
#' @export
#' @param object An instance of class `stanfit` produced by `hbam()`.
#' @param inc_stimuli Logical: Should estimated stimulus positions also be shown?
#' @param n_draws Integer specifying the number of posterior draws to use when illustrating the uncertainty of the population distribution. Defaults to 15.
#' @param color Color of lines illustrating uncertainty.
#' @param fill Fill color for density plots.
#' @param alpha_color Number in \[0,1\]: Inverse level of transparency for line color.
#' @param alpha_fill Number in \[0,1\]: Inverse level of transparency for fill color.
#' @param seed A positive integer specifying an optional seed for reproducibility. The seed is used to select respondent position draws for illustrating uncertainty.
#' @return A `ggplot` object.
#'

plot_respondents <- function(object, inc_stimuli = TRUE, n_draws = 15, color = "#053061", fill = "#2166AC", alpha_color = .6, alpha_fill = .7 / n_draws, seed = 1) {
  pd <- get_plot_data(object, n_draws = n_draws, seed = seed)
  suppressWarnings(
  if (inc_stimuli == T) {
    p <- ggplot2::ggplot() + ggplot2::geom_density(data = pd$chi_draws, ggplot2::aes(.data$chi, by = .data$draw_no), color = ggplot2::alpha(color, alpha_color), fill = ggplot2::alpha(fill, alpha_fill), na.rm = TRUE) +
      ggplot2::geom_text(data = pd$s_label, ggplot2::aes(x = .data$x, y = 0, label = .data$name), hjust = 0, nudge_y = .008, check_overlap = TRUE, angle = 90) +
      ggplot2::geom_point(data = pd$s_label, ggplot2::aes(x = .data$x, y = 0), color = color) +
      ggplot2::labs(x = "Ideological scale", y = "Density")
  } else {
    p <- ggplot2::ggplot() + ggplot2::geom_density(data = pd$chi_draws, ggplot2::aes(.data$chi, by = .data$draw_no), color = ggplot2::alpha(color, alpha_color), fill = ggplot2::alpha(fill, alpha_fill), na.rm = TRUE) +
      ggplot2::labs(x = "Ideological scale", y = "Posterior density")
  })
  lim <- max(abs(quantile(pd$chi_draws$chi, probs = c(.002, .998))))
  p <- p + xlim(-lim, lim)
  return(p)
}

#' Plot estimated stimulus positions
#'
#' Plot marginal posterior distributions of stimulus positions from an HBAM model
#'
#' @export
#' @param object An instance of class `stanfit` produced by `hbam()`.
#' @param rev_color Logical: Display low positions as red and high positions as blue.
#' @param alpha Number in \[0,1\]: Inverse level of transparency for fill color.
#' @return A `ggplot` object.
#'

plot_stimuli <- function(object, rev_color = FALSE, alpha = .55) {
  pd <- get_plot_data(object)
  pal <- RColorBrewer::brewer.pal(n = 11, name = "RdBu")
  if (rev_color == F) { pal <- rev(pal) }
  # Placing stimuli in 11 equally spaced categories for fill-color:
  cuts = (-5:6 - .5) * max(abs(pd$s_label$x)) * 2 / 10
  cats <- as.numeric(cut(pd$s_label$x, breaks = cuts))

  p <- ggplot2::ggplot(pd$s_draws, ggplot2::aes(.data$value, fill = .data$name, color = .data$name)) + ggplot2::geom_density(alpha = alpha, trim = TRUE, outline.type = "full") +
    ggplot2::geom_text(data = pd$s_label, ggplot2::aes(x = .data$x, y = 0, label = .data$name), hjust = 0, nudge_x = -.007,
              nudge_y = .025 * max(pd$s_label$mode_y), check_overlap = TRUE, angle = 90, color = "black") +
    ggplot2::labs(x = "Ideological scale", y = "Posterior density") +
    ggplot2::theme(legend.position = "none") + ggplot2::scale_fill_manual(values = pal[cats]) + ggplot2::scale_color_manual(values = colorspace::darken(pal[cats], .2))
  return(p)
}

# Function to draw from the scaled inverse chi-squared distribution:
rinvchisq <- function(n, df, scale = 1 / df) {
  (df * scale) / rchisq(n, df = df)
}

# Function to draw from the Dirichlet distribution:
rdirichlet <- function (n, alpha) {
  l <- length(alpha)
  x <- matrix(rgamma(l * n, alpha), ncol = l, byrow = TRUE)
  sm <- x %*% rep(1, l)
  return(x / as.vector(sm))
}

# Function to generate data:
create_data <- function(N = 500, J = 4, B = 3, nu = 8.4, tau = .19 * B, rho_a = 5, psi = .89,
                        sigma_a = .19 * B, sigma_b = .27, sigma_c = .52 * B,
                        seed = 1) {
  set.seed(seed)

  # Latent parameters:
  true_alpha <- rnorm(N, 0, sigma_a)
  true_beta0 <- exp(rnorm(N, 0, sigma_b))
  true_kappa <- rbinom(N, 1, psi)
  true_beta <- true_beta0 * true_kappa - true_beta0 * (1 - true_kappa)
  true_chi <- rnorm(N, 0, sigma_c)
  true_theta <- c(runif(1, -B + 1, 0), runif(1, 0, B - 1), runif(J - 2, -B + 1, B - 1))

  # Stimulus positions:
  true_rho <- rdirichlet(1, rep(rho_a, J))
  true_eta <- rinvchisq(N, nu, scale = (tau^2) * (J^2))
  sd_ij <- matrix(rep(sqrt(true_eta), times = J) * rep(true_rho, each = N), nrow = N, ncol = J)
  errors <- matrix(rnorm(N * J, 0, sd_ij), nrow = N, ncol = J)
  ppos <- round(matrix(rep(true_alpha, each = J) + (rep(true_beta, each = J) * true_theta),
                       N, J, byrow = TRUE) + errors)
  ppos[ppos < -B] <- -B; ppos[ppos > B] <- B

  # Self-placements:
  self <- round(true_alpha + true_beta * true_chi + rnorm(N, 0, sqrt(true_eta) * min(true_rho)))
  self[self < -B] <- -B; self[self > B] <- B

  dat <- prep_data(self, ppos, B = B)
  dat$true_theta <- true_theta
  dat$true_chi <- true_chi[dat$keep]
  dat$true_alpha <- true_alpha[dat$keep]
  dat$true_beta <- true_beta[dat$keep]
  dat$true_kappa <- true_kappa[dat$keep]

  return(dat)
}

inits_omni <- function(chain_id = 1, dat) {
  alpha_raw <- matrix(rnorm(2 * dat$N, 0, .3), ncol = 2)
  beta_raw <- matrix(rnorm(dat$N * 2, 0, .2), ncol = 2)
  if (dat$fixed == 0) {
    tau <- rinvchisq(1, 500, (dat$B / 3))
    dim(tau) <- 1
  } else {
    tau <- NULL
  }
  if (dat$bam == 0 & dat$fixed == 0) {
    theta_lr <- dat$mean_spos[c(dat$L, dat$R)] + c(runif(1, -dat$B / 10, 0), runif(1, 0, dat$B / 10))
    sigma_alpha <- rinvchisq(1, 200, .75)
    sigma_beta <- runif(1, .3, .35)
    dim(sigma_alpha) <- 1
    dim(sigma_beta) <- 1
  } else {
    theta_lr <- NULL
    sigma_alpha <- NULL
    sigma_beta <- NULL
  }
  if (dat$fixed == 1) {
    theta_lr <- dat$mean_spos[c(dat$L, dat$R)] + c(runif(1, -dat$B / 10, 0), runif(1, 0, dat$B / 10))
  }
  if (dat$flip == 0) {
    alpha_raw <- alpha_raw[, 1]
    dim(alpha_raw) <- c(dat$N, 1)
    beta_raw <- beta_raw[, 1]
    dim(beta_raw) <- c(dat$N, 1)
    lambda_raw <- NULL
    psi <- NULL
  } else {
    lambda_raw <- rnorm(dat$N, 0, .05)
    if (dat$fixed == 0) {
      psi <- exp(rnorm(1, 1.3, .1))
      dim(psi) <- 1
    } else {
      psi <- NULL
    }
  }
  if (dat$het == 1) {
    nu <- 3 + rinvchisq(1, 100, 7)
    eta <- rinvchisq(dat$N, 100, dat$J^2 * (dat$B / 3)^2)
    rho <- rdirichlet(1, rep(75, dat$J))
    dim(nu) <- 1
  } else {
    nu <- NULL
    eta <- NULL
    rho <- 1
    dim(rho) <- 1
  }
  if (dat$fixed == 1) {
    nu <- NULL
  }
  if (dat$group == 1) {
    mu_alpha_raw <- rdirichlet(1, rep(500, dat$G))
    mu_beta_raw <- rdirichlet(1, rep(500, dat$G))
  } else {
    mu_alpha_raw <- 1
    mu_beta_raw <- 1
    dim(mu_alpha_raw) <- 1
    dim(mu_beta_raw) <- 1
  }
  if (dat$rat == 1) {
    gamma <- runif(dat$N, 0, .3)
    gam_a <- runif(1, 2, 3)
    gam_b <- runif(1, 1.1, 1.3)
    dim(gam_a) <- 1
    dim(gam_b) <- 1
    zeta <- runif(dat$K, .1, .9)
  } else {
    gamma <- NULL
    gam_a <- NULL
    gam_b <- NULL
    zeta <- NULL
  }
  list (
    theta_raw = dat$mean_spos + rnorm(dat$J, 0, (dat$B / 5) * 0.25),
    theta_lr = theta_lr,
    sigma_alpha_par = sigma_alpha,
    alpha_raw = alpha_raw,
    sigma_beta_par = sigma_beta,
    beta_raw = beta_raw,
    nu_par = nu,
    tau_par = tau,
    eta = eta,
    rho = rho,
    lambda_raw = lambda_raw,
    psi_par = psi,
    mu_alpha_raw = mu_alpha_raw,
    mu_beta_raw = mu_beta_raw,
    gamma = gamma,
    gam_a = gam_a,
    gam_b = gam_b,
    zeta = zeta
  )
}

rinvchisq <- function(n, df, scale = 1/df) {
  return((df * scale) / rchisq(n, df = df))
}

rdirichlet <- function (n, alpha) {
  l <- length(alpha)
  x <- matrix(rgamma(l * n, alpha), ncol = l, byrow = TRUE)
  sm <- x %*% rep(1, l)
  out <- x / as.vector(sm)
  if (n == 1) { out <- as.numeric(out) }
  return(out)
}

---
title: "Hierarchical Bayesian Aldrich-McKelvey Scaling in R via Stan"
output: rmarkdown::html_vignette
bibliography: hbam_refs.bib 
csl: american-journal-of-political-science.csl
vignette: >
  %\VignetteIndexEntry{Hierarchical Bayesian Aldrich-McKelvey Scaling in R via Stan}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
resource_files:
  - p_stim.svg
  - p_resp.svg
  - p_trace_theta.svg
  - p_lambda.svg
  - p_eta.svg
  - p_chi.svg
  - p_alpha.svg
  - p_abs_beta.svg
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE)

load("data.rda")

library("dplyr")
```

```{=html}
<!-- avoid border around images -->
<style> img { border: 0; } </style>
```
The goal of the **hbamr** package is to enable users to efficiently perform Hierarchical Bayesian Aldrich-McKelvey (HBAM) scaling in R. Aldrich-McKelvey (AM) scaling is a method for estimating the ideological positions of survey respondents and political actors on a common scale using ideological survey data [@aldrich:1977]. The hierarchical versions of the AM model included in this package outperform other versions both in terms of yielding meaningful posterior distributions for all respondent positions and in terms of recovering true respondent positions in simulations. The original version of the default model is described in an [open access article](https://doi.org/10.1017/pan.2023.18) [@boelstad:2023].

The package mainly fits models via the NUTS algorithm in **rstan** -- a Markov chain Monte Carlo (MCMC) algorithm. However, it also offers a simplified model that can be fit using optimization to analyze large data sets quickly.

This vignette provides an overview of how to use the key functions in the **hbamr** package. The vignette walks through an applied example, showing how to prepare data, fit models, extract estimates, plot key results, and perform cross-validation.

## Example Data

For illustration, we will use data from the 1980 American National Election Study (ANES). This is the same data set that serves to illustrate the original AM model in the **basicspace** package. The data set is included in the **hbamr** package and can be loaded by running `data(LC1980)`.

The data set contains respondents' placements of themselves and six stimuli on 7-point Liberal-Conservative scales. The stimuli in question are: The Democratic and Republican parties, Democratic presidential candidate Jimmy Carter, Republican candidate Ronald Reagan, independent candidate (and former Republican) John B. Anderson, and Ted Kennedy (who challenged the incumbent Carter, but failed to win the Democratic nomination).

We load the data and re-code missing values as follows:

```{r load_data, echo = TRUE, eval = FALSE, include = TRUE}
library("hbamr")
data(LC1980)
LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA 
self <- LC1980[, 1]
stimuli <- LC1980[, -1]
```

```{r show_data, eval = FALSE}
head(stimuli) 
```

```{r, echo = FALSE}
head_stimuli 
```

## Preparing the Data

The function `prep_data()` serves to prepare the data. This function can be run ahead of fitting the models, or it can be run implicitly as part of a single function call to fit the models (as shown below). The function takes a vector of $N$ ideological self-placements and an $N \times J$ matrix of stimulus placements. The self-placement vector is required to estimate respondent positions, but can be dropped if such data are not available or if the respondent positions are not of interest. 

`prep_data()` applies a set of inclusion criteria, performs any necessary data transformation, and returns a list of data suited for sampling in **rstan**. The stimuli data are stored in a vector as a long-form sparse matrix. If the stimuli data include column-names, these will be preserved for later use.

Any missing data must be set to `NA` before use. The `prep_data()` function allows the user to decide how many missing values should be permitted per respondent by specifying the argument `allow_miss`. (The default is `allow_miss = 2`. Alternatively, the argument `req_valid` specifies how many valid observations to require per respondent. The default is `req_valid = J - allow_miss`, but, if specified, `req_valid` takes precedence.) Similarly, the user may specify how may unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is `req_unique = 2`, which means that respondents who place all stimuli in exactly the same place will not be included.

The data provided to `prep_data()` can be centered, but they do not have to be: The function will detect un-centered data and attempt to center these automatically, assuming that the highest and lowest observed values in the data mark the extremes of the scale.

To use the `prep_data()` function on the example data using the default settings, we would run:

```{r prep_data, echo = TRUE, eval = FALSE}
dat <- prep_data(self, stimuli)
```

Users who want to keep other covariates for subsequent analysis, may find it useful to run `prep_data()` separately from the call to fit the models. The list returned by this function includes the logical vector `keep`, which identifies the rows in the original data that have been kept. If we had a data set `x` containing covariates, and used `prep_data()` to produce the list `dat`, then we could use `x[dat$keep, ]` to get a subset of `x` corresponding to the data used in the analysis. (The order of the individuals/rows in the data remains unchanged by the functions in this package.)

## Models

This package provides several alternative models that can be selected using the names below. Users who are unsure which model to use are advised to use the default HBAM model. If speed or sampling diagnostics are an issue, HBAM_MINI may provide a useful alternative. (See also the section on cross-validation for further discussion on the issue of model selection.)

**HBAM** is the default model, which allows for scale flipping and employs hierarchical priors on the shift and stretch parameters. It also models heteroskedastic errors that vary by both individual and stimuli. Compared to the model in @boelstad:2023, this version has been slightly revised to provide faster sampling. A key difference from the original model is that the respondent positions are not treated as parameters, but rather calculated as a function of self-placements, individual-level parameters, and simulated errors. This makes the model considerably faster, while yielding very similar results. The model simulates errors in the self-placements of the same magnitude as that with which the respondent in question places the stimulus with the smallest errors. All models in the package use this approach.

**HBAM_MULTI** is a version that models differences between groups defined by the user. It requires a vector identifying the groups to be supplied as the argument `group_id`. The model gives each group separate hyperparameters for the locations of the prior distributions for the shift and stretch parameters. Rather than shrinking the estimates toward the mode for the whole dataset, this model shrinks the estimates toward the mode for the group. The vectors of hyperparameters are called `mu_alpha` and `mu_beta` and are constructed to have means of 0. The scales of the priors on these hyperparameters can be set by the user via the arguments `sigma_mu_alpha` and `sigma_mu_beta`. The default values are B / 10 and .2, respectively. (Here, B measures the length of the survey scale as the number of possible placements on one side of the center.) One potential use for this model is to supply self-placements as `group_id`, and thus give each self-placement group its own prior distribution for the shift and stretch parameters.

**HBAM_NF** (formerly HBAM_0) is a version of the HBAM model that does not allow for scale flipping. This may be useful if there are truly zero cases of scale flipping in the data. Such scenarios can be created artificially, but may also arise in real data. For example, expert surveys appear unlikely to contain many instances of scale flipping. For data that contain zero cases of flipping, models that allow for flipping contain superfluous parameters that lead to inefficient sampling. Models that do not allow for flipping will sample faster and typically yield slightly more accurate estimates. Such models are therefore usually preferable when no flipping is present.

**HBAM_MULTI_NF** is a version of the HBAM_MULTI model that does not allow for scale flipping.

**HBAM_MINI** is a version of the HBAM model that assumes the prediction errors in the stimuli placements to be homoskedastic. This model tends to sample faster than the standard HBAM model while yielding very similar point estimates. For large datasets, this model may provide a reasonable compromise between model complexity and estimation speed.

**FBAM** is a version of the HBAM model with fixed hyperparameters to allow fitting via optimization rather than MCMC -- which can be useful for large data sets. This model allows the user to specify the scales of the priors for the shift and (logged) stretch parameters via the arguments `sigma_alpha` and `sigma_beta`. The default values are B / 5 and .3, respectively. These defaults are intended to be realistic and moderately informative. Users who want to control the degree of shrinkage of the individual-level parameters may find it useful to fit this model -- or other FBAM models -- via either MCMC or optimization.

**FBAM_MULTI** is a version of the FBAM model that shares the group-modeling features of the HBAM_MULTI model. It allows the user to set the scales of the priors for the shift and stretch parameters via the arguments `sigma_alpha` and `sigma_beta`, and set the scales of the priors on `mu_alpha` and `mu_beta` via the arguments `sigma_mu_alpha` and `sigma_mu_beta`.

**FBAM_MULTI_NF** is a version of the FBAM_MULTI model that does not allow for scale flipping.

**HBAM_R_MINI** is a version of the HBAM_MINI model that incorporates the rationalization component of the ISR model in @boelstad:2020. This model requires additional data to be supplied as the argument `pref`: An N × J matrix of stimuli ratings from the respondents. The rationalization part of the model is simplified relative to the original ISR model: The direction in which respondents move disfavored stimuli is estimated as a common expectation for each possible self-placement on the scale.

**BAM** is an unpooled model with wide uniform priors on the shift and stretch parameters. It is similar to the JAGS version introduced by @hare:2015, although the version included here has been adjusted to yield stretch parameters with an average of approximately one (and thus produce a scale similar to those of the other models in the package). Like the other models, this version of the BAM model also simulates errors in the self-placements to yield a realistic level of uncertainty. While this model is simple and fast, it tends to overfit the data and produce invalid posterior distributions for some respondent positions [@boelstad:2023]. However, it could potentially be useful as a baseline for model comparisons in situations where respondent positions are not of interest.

**HBAM_2** has been replaced by the more general HBAM_MULTI model.

These models can also be used in situations where self-placements are not available and the only goal is to estimate stimulus positions or respondents' shift and stretch parameters. While the latent respondent positions will not be estimated, all other parameters are unaffected if the argument `self` is dropped when calling `prep_data()` or `hbam()`.

### Summary of Model Features

|                        | HBAM<br>  | HBAM<br>MULTI | HBAM<br>NF | HBAM<br>MULTI_NF | HBAM<br>MINI |
|-----------------------------|:------------:|:------------:|:------------:|:------------:|:------------:|
| Hierarchical priors    |     X     |       X       |     X      |        X         |      X       |
| Scale flipping         |     X     |       X       |            |                  |      X       |
| Group-differences      |           |       X       |            |        X         |              |
| Heteroskedastic errors |     X     |       X       |     X      |        X         |              |

: Recommended models for fitting via MCMC

|                            | FBAM | FBAM<br>MULTI | FBAM<br>MULTI_NF | HBAM<br>R_MINI | BAM<br>  |
|-----------------------------|:------------:|:------------:|:------------:|:------------:|:------------:|
| Hierarchical priors        |              |               |                  |       X        |          |
| User-defined priors\*      |      X       |       X       |        X         |                |          |
| Scale flipping             |      X       |       X       |                  |       X        |    X     |
| Group-differences          |              |       X       |        X         |                |          |
| Heteroskedastic errors     |      X       |       X       |        X         |                |    X     |
| Rationalization            |              |               |                  |       X        |          |
| Fitting via optimization\* |      X       |       X       |        X         |                |          |

: Non-hierarchical and special purpose models ("\*" marks optional features)

## Fitting

The `hbam()` function can be used to fit all models in this package and obtain a `stanfit` object. The default model is HBAM, while other models can be specified via the argument `model`. 

Unless the user supplies pre-prepared data via the `data`-argument, `hbam()` will implicitly run `prep_data()`. It therefore takes the same arguments as the `prep_data()` function (i.e. `self`, `stimuli`, `group_id`, `allow_miss`, `req_valid`, and `req_unique`).

To fit the HBAM model using the default settings, we would run:

```{r fithbam1, eval = FALSE}
fit_hbam <- hbam(self, stimuli)
```

To fit the HBAM_MINI model while requiring complete data for all respondents, we would run:

```{r fitbam, eval = FALSE, cache = FALSE, message = FALSE, warning = FALSE}
fit_hbam_mini <- hbam(self, stimuli, model = "HBAM_MINI", allow_miss = 0)
```

If we wanted to run the `prep_data()` function separately before fitting the model, we would supply output from `prep_data()` as the argument `data`:

```{r fithbam2, cache = FALSE, eval = FALSE, message = FALSE, warning = FALSE}
dat <- prep_data(self, stimuli, allow_miss = 0) 
fit_hbam <- hbam(data = dat)
```

```{r prep_data3, include = FALSE, eval = FALSE}
dat <- prep_data(self, stimuli, allow_miss = 0) 
```

To fit the HBAM_MULTI or FBAM_MULTI model, we would need to supply a vector identifying the groups of interest. One option would be to supply respondents' self-placements as `group_id` and thus give each self-placement group their own prior distributions for the shift and stretch parameters. If we decided to fit the FBAM_MULTI model, we could also specify our own scales for the priors on key parameters:

```{r, eval = FALSE}
fit_fbam_multi <- hbam(self, stimuli, model = "FBAM_MULTI", group_id = self, 
                       sigma_alpha = .8, sigma_mu_alpha = .7, 
                       sigma_beta = .4, sigma_mu_beta = .25)
```

`hbam()` uses `rstan::sampling()`, and any additional arguments to `hbam()` will be passed on to the sampling function. By default, `hbam()` will run 4 chains and detect the number of available CPU cores. If possible, `hbam()` will use as many cores as there are chains. The other sampling defaults are: `warmup = 1000` and `iter = 2000`. These settings can all be overridden in the `hbam()` call.

### Fitting via Optimization

For very large data sets, optimization can be a useful and much faster alternative to MCMC. However, the optimization feature provided in this package only provides maximum a posteriori (MAP) point estimates.

The `fbam()` function fits FBAM models using `rstan::optimizing()`. The `fbam()` function works just like the `hbam()` function, except the arguments for `rstan::sampling()` do not apply. To fit the FBAM model using default settings, we would run:

```{r fitfbam, eval = FALSE}
fit_fbam <- fbam(self, stimuli)
```

### Execution Times

Models of the kind included in the **hbamr** package face an inevitable trade-off between nuance (i.e. model complexity) and execution times. For small data sets (like ANES 1980), all models provide reasonable running times, but for large data sets, more complex models tend to get slow. In these cases, the HBAM_MINI model may be a useful alternative. For very large data sets, fitting the FBAM model via optimization may be the best alternative.

|                                   | HBAM <br>iter=2000 | HBAM_NF <br>iter=2000 | HBAM_MINI <br>iter=2000 | FBAM<br>iter=2000 | FBAM <br>optimization |
|--------------|:----------:|:----------:|:----------:|:----------:|:----------:|
| **ANES 1980** <br>  (N=643, J=6)  |          1m          |          25s          |          50s           |          1m          |             3s             |
| **ANES 2012** <br>  (N=4949, J=4) |          10m         |           3m 40s        |            8m            |           10m            |            12s             |

: Execution times on an Apple M4 Pro CPU

## Plotting

The **hbamr** package contains several functions for creating presentable plots of the results. The package uses **ggplot2**, which means ggplot-themes can be added to the plots. 

### Stimuli Positions

The function `plot_stimuli()` plots the marginal posterior distributions of all stimuli in the data. By default, it will fill the distributions with shades from blue to red depending on the position on the scale. The argument `rev_color = TRUE` will reverse the order of the colors.

```{r, include = TRUE, eval = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"}
plot_stimuli(fit_hbam)
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"}
knitr::include_graphics("p_stim.svg")
```

In this example, we see that John B. Anderson -- the former Republican who ran as an independent candidate -- gets a wider posterior distribution, suggesting that voters were more uncertain about where to place him relative to the others.

### Respondent Positions

The function `plot_respondents()` plots the distribution of estimated respondent positions. It illustrates the uncertainty of the estimates by calculating the population density for each of a set of posterior draws. The default is to use 15 draws for each respondent, but this can be altered by specifying the argument `n_draws`. The `plot_respondents()` function also plots the estimated stimulus positions by default, but this behavior can be turned off by adding the argument `inc_stimuli = FALSE`.

```{r, include = TRUE, eval = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"}
plot_respondents(fit_hbam, n_draws = 10)
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .4, fig.width = 8, out.width = "100%"}
knitr::include_graphics("p_resp.svg")
```

Users who want to customize the plots further can obtain the underlying data by using the function `get_plot_data()`. This function accepts the same `n_draws`-argument as `plot_respondents()`. The output is a list of three tibbles: The first element contains the posterior mean stimulus positions, as well as the $x$- and $y$-values of the posterior modes (which can be useful for labeling the distributions). The second element contains the posterior draws for the stimulus positions (which can be used to calculate marginal posterior densities). The third element contains the selected number of posterior draws for each respondent (which form the key ingredient for the `plot_respondents()` function).

```{r get_plot_dat, include = TRUE, eval = FALSE, include= FALSE, eval = FALSE}
get_plot_data(fit_hbam)
```

### Individual Parameters over Self-Placements

The function `plot_over_self()` plots the distributions of key parameter estimates over the respondents' self-placements. The function will accept either a single `stanfit` object produced by `hbam()`, results from `fbam()`, or a list of such objects. 

The user specifies which parameter to show via the argument `par`. This can be either of the following: `"alpha"`, `"beta"`, `"abs_beta"`, `"lambda"`, or `"chi"`, where `"abs_beta"` calls for the absolute value of beta to be used. By default, the function uses posterior median estimates, but this can be changed by specifying `estimate = "mean"`.

#### Shifting

To compare the distributions of estimated shift parameters from the HBAM and HBAM_MINI models, we would run:

```{r plot_alpha, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
plot_over_self(list(fit_hbam, fit_hbam_mini), "alpha")
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
knitr::include_graphics("p_alpha.svg")
```

#### Stretching

For models that allow for scale flipping, the draws for $\beta$ combine the separate parameters for each flipping-state. The absolute value of $\beta$ may therefore be better suited for examining the extent to which each individual stretches the ideological space. To inspect the distribution of these values across self-placements, we would run:

```{r plot_beta, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
plot_over_self(list(fit_hbam, fit_hbam_mini), "abs_beta")
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
knitr::include_graphics("p_abs_beta.svg")
```

The pattern above, where respondents with more extreme self-placements have more extreme $\beta$ parameters, is exactly the kind of differential item functioning that the models in this package are intended to correct for: These respondents tend to place both stimuli and themselves further out on the scale than others do, thus appearing more extreme in comparison.

#### Flipping

To see whether the $\beta$ parameters are likely to be positive or negative, we can look at the expectations of the flipping parameters, $\lambda$. These parameters represent each respondent's probability of *not* flipping the scale:

```{r plot_lambda, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
plot_over_self(list(fit_hbam, fit_hbam_mini), "lambda")
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
knitr::include_graphics("p_lambda.svg")
```

In this example, flipping is uncommon, but respondents who place themselves in the middle have a somewhat higher flipping-probability. This may suggest that some of these respondents are less informed about politics and provide less accurate answers.

#### Respondent Positions

It may also be useful to inspect the distribution of the scaled respondent positions over the self-placements. This illustrates the extent to which the model has transformed the original data. In this example, the impact of the models is generally modest, although a few respondents have been detected as having flipped the scale, and thus have had their self-placement flipped back.

```{r plot_chi, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
plot_over_self(list(fit_hbam, fit_hbam_mini), "chi")
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .37, fig.width = 8.5, fig.align = "center", out.width = "100%"}
knitr::include_graphics("p_chi.svg")
```

#### Additional Parameters

Other individual-level parameters like `"eta"` can also be plotted if these have been passed to `hbam()` via the argument `extra_pars` when fitting the model. Parameters like $\eta$ are not stored in the `hbam()` results by default, as this would increase the post-processing time as well as the size of the model fits. (Note also that homoskedastic models have no `"eta"` parameters and "NF"-type models have no `"lambda"` or `"kappa"` parameters.) 

The estimated $\eta$ parameters yield information about the accuracy of respondents' answers. When the argument `par = "eta"` is specified, the plotting function will display $\sqrt{\eta_i} / J$, which equals the average error for each individual (the mean of $\sigma_{ij}$ for each $i$ across $j$). The point estimates will still be calculated using the posterior median, unless the argument `estimate = "mean"` is added.

```{r plot_eta, include = TRUE, eval = FALSE, echo = TRUE, fig.asp = .67, fig.width = 4.25, fig.align = "center", out.width = "50%"}
fit_hbam <- hbam(data = dat, extra_pars = "eta")
plot_over_self(fit_hbam, "eta")
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .67, fig.width = 4.25, fig.align = "center", out.width = "50%"}
knitr::include_graphics("p_eta.svg")
```

## Posterior Summaries

The package also contains a wrapper for `rstan::summary()` called `get_est()`. This function takes the arguments `object` -- a `stanfit` object produced by `hbam()` or a list produced by `fbam()` -- and `par` -- the name of the parameter(s) to be summarized. The function returns a tibble, which by default contains the posterior mean, the 95% credible interval, the posterior median, the estimated number of effective draws, and the split R-hat. One can obtain other posterior quantiles by using the argument `probs`. To get a 50% credible interval (and no median), one would add the argument `probs = c(0.25, 0.75)`. To include the Monte Carlo standard error and the posterior standard deviation, use the argument `simplify = FALSE`. (When applied to outputs from `fbam()`, `get_est()` only returns point estimates.)

The posterior draws for the stimulus positions can be summarized as follows:

```{r get_est_t, include = TRUE, eval = FALSE}
get_est(fit_hbam, "theta")
```

```{r, echo = FALSE}
est_theta
```

To get summaries of the respondent positions, we would specify `par = "chi"`. If we want the results to match the rows in the original dataset by reporting rows of NAs for respondents who were not included in the analysis, we would add the argument `format_orig = TRUE`:

```{r get_est_v, include = TRUE, eval = FALSE}
get_est(fit_hbam, "chi", format_orig = TRUE)
```

```{r, echo = FALSE}
est_chi_orig
```

## Cross-Validation

A useful way to compare alternative models is to estimate their out-of-sample prediction accuracy. More specifically, we can estimate their expected log pointwise predictive density for a new data set (ELPD). 

The **rstan** and **loo** packages contain functions for estimating ELPDs by performing approximate leave-one-out (LOO) cross-validation using Pareto smoothed importance sampling (PSIS). These functions can be used on model fits from `hbam()` if the argument `extra_pars = "log_lik"` is specified when fitting the model:

```{r, eval = FALSE}
fit_hbam <- hbam(data = dat, extra_pars = "log_lik")
loo_hbam <- loo::loo(fit_hbam)
```

However, PSIS-LOO only works when all Pareto *k* values are sufficiently low, or when the number of high values is so low that moment matching can be used on the problematic cases. This is not always the case for the models in this package. 

The **hbamr** package therefore allows users to perform *K*-fold cross-validation. The function `hbam_cv()` is similar to `hbam()` and takes the same arguments, but will perform a *K*-fold cross-validation for the chosen model. In contrast to `hbam()`, the default for `hbam_cv()` is to not allow respondents to have any missing values (`allow_miss = 0`). The reason is that the cross-validation essentially creates missing values by excluding some data from each run. 

A key choice when performing cross-validation is to set *K* (the number of folds to use), and the default in this package is 10. To preserve memory, the function extracts the summaries of the log-likelihoods for the held-out data and drops the stanfit objects once this is done. The memory requirements of the function is therefore similar to running a single analysis with one chain per core. The function that splits the data into *K* folds uses a default seed to produce the same folds each time, unless a different seed is specified.

The `hbam_cv()` function is written to allow parallel computation via the `future` package to minimize execution time. The `future` package offers several computational strategies, of which "multisession" works on all operating systems. It most settings, it is advisable to use all physical CPU cores when performing cross-validation. To set up parallel computation using 4 cores via the `future` package, we could run:

```{r, eval = FALSE}
library(future)
plan(multisession, workers = 4)
```

To perform 10-fold cross-validation for a selection of models, we could run:

```{r, include = FALSE}
load("elpds.rda")
```

```{r run_cv_new, eval = FALSE}
kfold_bam <- hbam_cv(self, stimuli, model = "BAM")
kfold_hbam <- hbam_cv(self, stimuli, model = "HBAM")
kfold_hbam_nf <- hbam_cv(self, stimuli, model = "HBAM_NF")
kfold_hbam_multi <- hbam_cv(self, stimuli, group_id = self, 
                            model = "HBAM_MULTI")
```

The `hbam_cv()` function returns an object of classes `kfold` and `loo`, which can be further processed using the `loo` package. To compare the estimated ELPDs, we could run:

```{r, eval = FALSE}
print(loo::loo_compare(list(BAM = kfold_bam, 
                            HBAM = kfold_hbam, 
                            HBAM_NF = kfold_hbam_nf, 
                            HBAM_MULTI = kfold_hbam_multi)), simplify = FALSE)
```

```{r, echo = FALSE, message = FALSE, warning = FALSE}
library(loo)
print(loo1, simplify = FALSE)
```

We see that the unpooled BAM model is worse at predicting out-of-sample data than the other models, suggesting it overfits the data. The HBAM_NF model -- which does not allow for scale flipping -- also performs worse, suggesting it is too restrictive and underfits. The HBAM and HBAM_MULTI models outperform the other two models in this case. The HBAM_MULTI model -- with self-placements as `group_id` -- may have a slight edge, but the default HBAM model performs about equally well. 

We could also perform cross-validation for a model that does not account for heteroskedastic errors:

```{r, eval = FALSE}
kfold_hbam_mini <- hbam_cv(self, stimuli, model = "HBAM_MINI")
```

```{r, eval = FALSE}
kfold_hbam_mini
```

```{r, echo = FALSE}
kfold_hbam_mini
```

The ELPD for the HBAM_MINI model is notably lower than that of the HBAM model, which suggest there is a considerable degree of heteroskedasticity in the data. While modeling the heteroskedasticity increases prediction accuracy, it should be noted that this does not necessarily translate into much more accurate estimates of key model outputs. As shown in section on plotting, the results for the HBAM and HBAM_MINI models are very similar. In fact, their estimated respondent positions correlate at .97 and .99, depending on whether we use the posterior means or medians. Their estimated stimulus positions also correlate at .99, but this masks the fact that the HBAM_MINI model places the less well known candidate John B. Anderson further to the left than the HBAM model does. There are some subtle differences that users should be aware of, even if these models tend to produce very similar results.

## Diagnostics

The `rstan::sampling()` function that `hbam()` uses automatically performs a number of key diagnostic checks after sampling and issues warnings when a potential issue is detected. The authors of **rstan** emphasize diagnostics and careful model development, and users of **rstan** will more frequently encounter warnings than users of **rjags**. One warning users of this package may encounter is that the Bulk or Tail Effective Sample Size (ESS) is too low (see <https://mc-stan.org/misc/warnings.html>). The most straightforward solution to this issue is to increase the number of posterior draws, using the `iter` argument. However, this increases the computational load, and users should consider carefully what level of accuracy they need.

Because the `hbam()` function returns a `stanfit` object, the model fit can be examined using the full range of diagnostic tools from the **rstan** package. Users should consult the **rstan** documentation for details on the various diagnostic tests and plots that are available. One example of the available tools is `traceplot()`:

```{r traceplot_code, eval = FALSE, include = TRUE}
rstan::traceplot(fit_hbam, pars = "theta")
```

```{r, echo = FALSE, fig.align = "center", fig.asp = .5, fig.width = 8, fig.align = "center", out.width = "100%", warning = FALSE, message = FALSE}
knitr::include_graphics("p_trace_theta.svg")
```

### Limits to Exact Replication

The functions in this package accept a seed argument for all operations that involve a random number generator. Supplying a seed is sufficient to get the exact same results in repeated runs on the same system, but it does not ensure exact replication across systems or software versions. The [Stan Reference Manual](https://mc-stan.org/docs/reference-manual/reproducibility.html) explains why:

> Floating point operations on modern computers are notoriously difficult to replicate because the fundamental arithmetic operations, right down to the IEEE 754 encoding level, are not fully specified. The primary problem is that the precision of operations varies across different hardware platforms and software implementations.
>
> Stan is designed to allow full reproducibility. However, this is only possible up to the external constraints imposed by floating point arithmetic.

In short, running the functions in this package on different systems will likely yield slightly different results. It should be noted, however, that the differences across systems will be minimal and substantively negligible as long as the user obtains a sufficient number of posterior draws.

## References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_data.R
\name{prep_data}
\alias{prep_data}
\title{Prepare data to fit an HBAM or FBAM model}
\usage{
prep_data(
  self = NULL,
  stimuli,
  prefs = NULL,
  allow_miss = 2,
  req_valid = NA,
  req_unique = 2,
  B = NULL,
  group_id = NULL,
  quiet = FALSE
)
}
\arguments{
\item{self}{An optional numerical vector of N ideological self-placements. Any missing data must be coded as NA. If this argument is not supplied, respondent positions will not be estimated.}

\item{stimuli}{An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA.}

\item{prefs}{An N × J matrix of numerical stimulus ratings or preference scores. These data are only required by the \code{"HBAM_R_MINI"} model and will be ignored when fitting other models.}

\item{allow_miss}{Integer specifying how many missing stimulus positions should be accepted for an individual still to be included in the analysis. Defaults to 2.}

\item{req_valid}{Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is \code{req_valid = J - allow_miss}, but if specified, \code{req_valid} takes precedence.}

\item{req_unique}{Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is \code{req_unique = 2}.}

\item{B}{Scalar specifying the upper bound of the survey scale after centering. If not supplied, this information will be inferred from the data.}

\item{group_id}{Vector of length N identifying which group each respondent belongs to. The format can be factor, character, integer, or numeric. Respondents with NAs on \code{group_id} will be dropped when \code{group_id} is supplied. These data are only required by models with \code{"MULTI"} in their name and will be ignored when fitting other models.}

\item{quiet}{Logical: Should information about the data be printed to the console? Defaults to \code{FALSE}.}
}
\value{
A list of data to be used by \code{hbam()} or \code{fbam()}. The returned list includes the logical vector \code{keep}, which identifies the rows in the original data that have been kept for further analysis. The stimuli data are stored in a vector as a long-form sparse matrix. If the stimuli data include column-names, these will be preserved for later use.
}
\description{
This function prepares data to fit a hierarchical Bayesian Aldrich-McKelvey (HBAM) model. It can be run ahead of fitting the models, or it can be run implicitly as part of a single function call to fit the models using \code{hbam()} or \code{fbam()}. It applies a set of inclusion criteria, performs any necessary data transformation, and returns a list of data suited for sampling in \code{rstan}. The data provided to \code{prep_data()} can be centered, but they do not have to be: The function will detect un-centered data and attempt to center these automatically, assuming that the highest and lowest observed values in the data mark the extremes of the scale.
}
\examples{
# Loading and re-coding ANES 1980 data:
data(LC1980)
LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
self <- LC1980[, 1]
stimuli <- LC1980[, -1]

# Prepare data for model fitting, using defaults:
dat <- prep_data(self, stimuli)

# Prepare data for model fitting, using using alternative settings:
dat2 <- prep_data(self, stimuli, allow_miss = 0, req_unique = 3)

# Obtain the data that are included in the analysis:
self2 <- self[dat2$keep]
stimuli2 <- stimuli[dat2$keep, ]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LC1980.R
\docType{data}
\name{LC1980}
\alias{LC1980}
\title{1980 Liberal-Conservative Scales}
\format{
An object of class \code{matrix} (inherits from \code{array}) with 888 rows and 7 columns.
}
\source{
American National Election Studies.

This dataset was originally part of the \code{basicspace} package under the same name ("LC1980").
}
\usage{
data(LC1980)
}
\description{
Liberal-Conservative 7-point scales from the 1980 National Election Study.
Includes (in order) self-placement, and rankings of Carter, Reagan, Kennedy,
Anderson, Republican party, Democratic Party. Stored as a matrix of integers.
The numbers 0, 8, and 9 are considered to be missing values.
}
\examples{
data(LC1980)
LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
head(LC1980)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_stimuli.R
\name{plot_stimuli}
\alias{plot_stimuli}
\title{Plot estimated stimulus positions}
\usage{
plot_stimuli(object, rev_color = FALSE, alpha = 0.55)
}
\arguments{
\item{object}{An instance of class \code{stanfit} produced by \code{hbam()}.}

\item{rev_color}{Logical: Display low positions as red and high positions as blue.}

\item{alpha}{Number in [0,1]: Inverse level of transparency for fill color.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot marginal posterior distributions of stimulus positions from an HBAM model
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hbamr-package.R
\docType{package}
\name{hbamr-package}
\alias{hbamr-package}
\alias{hbamr}
\title{Hierarchical Bayesian Aldrich-McKelvey Scaling via Stan}
\description{
Fit hierarchical Bayesian Aldrich-McKelvey (HBAM) models using a form of Hamiltonian Monte Carlo via Stan. Aldrich-McKelvey (AM) scaling is a method for estimating the ideological positions of survey respondents and political actors on a common scale using positional survey data. The hierarchical versions of the Bayesian AM model included in this package outperform other versions both in terms of yielding meaningful posterior distributions for respondent positions and in terms of recovering true respondent positions in simulations. The package contains functions for preparing data, fitting models, extracting estimates, plotting key results, and comparing models using cross-validation.
}
\references{
\itemize{
\item Bølstad, Jørgen (2024). Hierarchical Bayesian Aldrich-McKelvey Scaling. \emph{Political Analysis}. 32(1): 50–64. \doi{10.1017/pan.2023.18}.
\item Stan Development Team (2024). RStan: the R interface to Stan. \url{https://mc-stan.org}.
}
}
\seealso{
\itemize{\item \url{https://jbolstad.github.io/hbamr/}}
}
\author{
Jørgen Bølstad
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hbam.R
\name{hbam}
\alias{hbam}
\title{Fit an HBAM model}
\usage{
hbam(
  self = NULL,
  stimuli = NULL,
  model = "HBAM",
  allow_miss = 2,
  req_valid = NA,
  req_unique = 2,
  prefs = NULL,
  group_id = NULL,
  data = NULL,
  pars = c("alpha", "beta", "chi", "lambda", "theta"),
  extra_pars = NULL,
  include = TRUE,
  chains = 4,
  cores = parallel::detectCores(logical = FALSE),
  warmup = 1000,
  iter = 2000,
  seed = sample.int(.Machine$integer.max, 1),
  control = list(max_treedepth = 7),
  sigma_alpha = NULL,
  sigma_beta = 0.3,
  sigma_mu_alpha = NULL,
  sigma_mu_beta = 0.2,
  ...
)
}
\arguments{
\item{self}{An optional numerical vector of N ideological self-placements. Any missing data must be coded as NA. If this argument is not supplied (either here or in a previous call to \code{prep_data()}), respondent positions will not be estimated. If the data have been prepared in advance via the \code{prep_data()} function, the argument supplied here will be ignored.}

\item{stimuli}{An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{model}{Character: Name of the model to be used. Defaults to HBAM. The available models are described under Details.}

\item{allow_miss}{Integer specifying how many missing stimulus positions should be accepted for an individual still to be included in the analysis. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function. Defaults to 2.}

\item{req_valid}{Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is \code{req_valid = J - allow_miss}, but if specified, \code{req_valid} takes precedence. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{req_unique}{Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is \code{req_unique = 2}. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{prefs}{An N × J matrix of numerical stimulus ratings or preference scores. These data are only required by the HBAM_R_MINI model and will be ignored when fitting other models.}

\item{group_id}{Vector of length N identifying which group each respondent belongs to. The format can be factor, character, integer, or numeric. Respondents with NAs on \code{group_id} will be dropped when \code{group_id} is supplied. These data are only required by models with \code{"MULTI"} in their name and will be ignored when fitting other models.}

\item{data}{List of data that have been prepared in advance via the \code{prep_data()} function. Not required if the arguments \code{self} and \code{stimuli} are provided.}

\item{pars}{A vector of character strings specifying parameters of interest. If \code{include = TRUE}, only samples for parameters named in pars are stored in the fitted results. Conversely, if \code{include = FALSE}, samples for all parameters except those named in pars are stored in the fitted results. The default is to store results for "alpha", "beta", "chi", "lambda", and "theta".}

\item{extra_pars}{A vector of character strings specifying parameters or generated quantities to be added to \code{pars}. This makes it easy to add one or several additional parameters of interest, without having to repeat the default \code{pars} vector. The default is \code{NULL}.}

\item{include}{Logical scalar defaulting to \code{TRUE} indicating whether to include or exclude the parameters given by the pars argument. If \code{FALSE}, only entire multidimensional parameters can be excluded, rather than particular elements of them.}

\item{chains}{A positive integer specifying the number of Markov chains. Defaults to 4.}

\item{cores}{The number of cores to use when executing the Markov chains in parallel. By default, all detected physical cores will be used if \code{chains} is equal to or higher than the number of cores.}

\item{warmup}{A positive integer specifying the number of warmup (aka burn-in) iterations per chain. If step-size adaptation is on (which it is by default), this also controls the number of iterations for which adaptation is run (and hence these warmup samples should not be used for inference). The number of warmup iterations should be smaller than \code{iter}.}

\item{iter}{A positive integer specifying the number of iterations for each chain (including warmup).}

\item{seed}{A positive integer specifying an optional seed for reproducibility. If this argument is not supplied, a random seed will be generated and the function will produce slightly different results on each run.}

\item{control}{A named list of parameters to control the sampler's behavior. See the documentation for \code{rstan::stan} for more details.}

\item{sigma_alpha}{A positive numeric value specifying the standard deviation of the prior on the shift parameters in the FBAM model, or the standard deviation of the parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to B / 5, where B measures the length of the survey scale as the number of possible placements on one side of the center.}

\item{sigma_beta}{A positive numeric value specifying the standard deviation of the prior on the logged stretch parameters in the FBAM model, or the standard deviation of the logged parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to .3.}

\item{sigma_mu_alpha}{A positive numeric value specifying the standard deviation of the prior on the group-means of the shift parameters in MULTI-type models. Defaults to B / 10.}

\item{sigma_mu_beta}{A positive numeric value specifying the standard deviation of the prior on the group-means of the logged stretch parameters in MULTI-type models. Defaults to .2.}

\item{...}{Arguments passed to \code{rstan::sampling()}.}
}
\value{
An object of S4 class \code{stanfit}.
}
\description{
Fit a Hierarchical Bayesian Aldrich-McKelvey model using automatically tuned Hamiltonian Monte Carlo sampling (NUTS) via \code{rstan}.
}
\details{
This package provides several alternative models that can be selected using the names below. Users who are unsure which model to use are advised to use the default HBAM model. If speed or sampling diagnostics are an issue, HBAM_MINI may provide a useful alternative.

\strong{HBAM} is the default model, which allows for scale flipping and employs hierarchical priors on the shift and stretch parameters. It also models heteroskedastic errors that vary by both individual and stimuli. Compared to the model in Bølstad (2024), this version has been slightly revised to provide faster sampling. A key difference from the original model is that the respondent positions are not treated as parameters, but rather calculated as a function of self-placements, individual-level parameters, and simulated errors. This makes the model considerably faster, while yielding very similar results. The model simulates errors in the self-placements of the same magnitude as that with which the respondent in question places the stimulus with the smallest errors. All models in the package use this approach.

\strong{HBAM_MULTI} is a version that models differences between groups defined by the user. It requires a vector identifying the groups to be supplied as the argument \code{group_id}. The model gives each group separate hyperparameters for the locations of the prior distributions for the shift and stretch parameters. Rather than shrinking the estimates toward the mode for the whole dataset, this model shrinks the estimates toward the mode for the group. The vectors of hyperparameters are called \code{mu_alpha} and \code{mu_beta} and are constructed to have means of 0. The scales of the priors on these hyperparameters can be set by the user via the arguments \code{sigma_mu_alpha} and \code{sigma_mu_beta}. The default values are B / 10 and .2, respectively. (Here, B measures the length of the survey scale as the number of possible placements on one side of the center.) One potential use for this model is to supply self-placements as \code{group_id}, and thus give each self-placement group its own prior distribution for the shift and stretch parameters.

\strong{HBAM_NF} (formerly HBAM_0) is a version of the HBAM model that does not allow for scale flipping. This may be useful if there are truly zero cases of scale flipping in the data. Such scenarios can be created artificially, but may also arise in real data. For example, expert surveys appear unlikely to contain many instances of scale flipping. For data that contain zero cases of flipping, models that allow for flipping contain superfluous parameters that lead to inefficient sampling. Models that do not allow for flipping will sample faster and typically yield slightly more accurate estimates. Such models are therefore usually preferable when no flipping is present.

\strong{HBAM_MULTI_NF} is a version of the HBAM_MULTI model that does not allow for scale flipping.

\strong{HBAM_MINI} is a version of the HBAM model that assumes the prediction errors in the stimuli placements to be homoskedastic. This model tends to sample faster than the standard HBAM model while yielding very similar point estimates. For large datasets, this model may provide a reasonable compromise between model complexity and estimation speed.

\strong{FBAM} is a version of the HBAM model with fixed hyperparameters to allow fitting via optimization rather than MCMC -- which can be useful for large data sets. This model allows the user to specify the scales of the priors for the shift and (logged) stretch parameters via the arguments \code{sigma_alpha} and \code{sigma_beta}. The default values are B / 5 and .3, respectively. These defaults are intended to be realistic and moderately informative. Users who want to control the degree of shrinkage of the individual-level parameters may find it useful to fit this model -- or other FBAM models -- via either MCMC or optimization.

\strong{FBAM_MULTI} is a version of the FBAM model that shares the group-modeling features of the HBAM_MULTI model. It allows the user to set the scales of the priors for the shift and stretch parameters via the arguments \code{sigma_alpha} and \code{sigma_beta}, and set the scales of the priors on \code{mu_alpha} and \code{mu_beta} via the arguments \code{sigma_mu_alpha} and \code{sigma_mu_beta}.

\strong{FBAM_MULTI_NF} is a version of the FBAM_MULTI model that does not allow for scale flipping.

\strong{HBAM_R_MINI} is a version of the HBAM_MINI model that incorporates the rationalization component of the ISR model by Bølstad (2020). This model requires additional data to be supplied as the argument \code{prefs}: An N × J matrix of stimuli ratings from the respondents. The rationalization part of the model is simplified relative to the original ISR model: The direction in which respondents move disfavored stimuli is estimated as a common expectation for each possible self-placement on the scale.

\strong{BAM} is an unpooled model with wide uniform priors on the shift and stretch parameters. It is similar to the JAGS version introduced by Hare et al. (2015), although the version included here has been adjusted to yield stretch parameters with an average of approximately one (and thus produce a scale similar to those of the other models in the package). Like the other models, this version of the BAM model also simulates errors in the self-placements to yield a realistic level of uncertainty. While this model is simple and fast, it tends to overfit the data and produce invalid posterior distributions for some respondent positions (see Bølstad 2024). However, it could potentially be useful as a baseline for model comparisons in situations where respondent positions are not of interest.

\strong{HBAM_2} has been replaced by the more general HBAM_MULTI model.

These models can also be used in situations where self-placements are not available and the only goal is to estimate stimulus positions or respondents' shift and stretch parameters. While the latent respondent positions will not be estimated, all other parameters are unaffected if the argument \code{self} is dropped when calling \code{prep_data()} or \code{hbam()}.

See the \code{hbamr} vignette for a table summarizing the key characteristics of the available models.
}
\examples{
\donttest{
# Loading and re-coding ANES 1980 data:
data(LC1980)
LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA

# Making a small subset of the data for illustration:
self <- LC1980[1:100, 1]
stimuli <- LC1980[1:100, -1]

# Fitting the HBAM_MINI model, obtaining 1000 draws:
fit_hbam_mini <- hbam(self, stimuli, model = "HBAM_MINI",
                   warmup = 500, iter = 1000, chains = 2)

# Preparing the data before fitting, requiring complete responses:
dat <- prep_data(self, stimuli, allow_miss = 0)
fit_hbam_mini <- hbam(data = dat, model = "HBAM_MINI",
                   warmup = 500, iter = 1000, chains = 2)

# Obtaining posterior summaries for the latent stimulus positions:
theta_est <- get_est(fit_hbam_mini, par = "theta")

# Obtaining posterior summaries for the latent respondent positions
  # in a format matching the rows in the original dataset:
chi_est <- get_est(fit_hbam_mini, par = "chi", format_orig = TRUE)

# Fitting the FBAM_MULTI_NF model with self-placements as group_id:
fit_fbam_multi_nf <- hbam(self, stimuli, group_id = self, model = "FBAM_MULTI_NF",
                       warmup = 500, iter = 1000, chains = 2)
}
}
\references{
\itemize{
\item Bølstad, Jørgen (2024). Hierarchical Bayesian Aldrich-McKelvey Scaling. \emph{Political Analysis}. 32(1): 50–64. \doi{10.1017/pan.2023.18}.
\item Bølstad, Jørgen (2020). Capturing Rationalization Bias and Differential Item Functioning: A Unified Bayesian Scaling Approach. \emph{Political Analysis} 28(3): 340–355.
\item Hare, Christopher et al. (2015). Using Bayesian Aldrich-McKelvey Scaling to Study Citizens' Ideological Preferences and Perceptions. \emph{American Journal of Political Science} 59(3): 759–774.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_est.R
\name{get_est}
\alias{get_est}
\title{Extract point estimates or other summaries of marginal posterior distributions}
\usage{
get_est(
  object,
  par = "theta",
  format_orig = FALSE,
  probs = c(0.025, 0.5, 0.975),
  simplify = TRUE,
  ...
)
}
\arguments{
\item{object}{An instance of class \code{stanfit} produced by \code{hbam()}, or a list produced by \code{fbam()}.}

\item{par}{Character: Name of the parameter type to be extracted. Typically \code{"theta"} (stimuli positions) or \code{"chi"} (respondent positions).}

\item{format_orig}{Logical: Should individual-level parameters be mapped to the original dataset by returning rows of NAs for respondents who were not included in the analysis? Defaults to \code{FALSE}.}

\item{probs}{A numeric vector of quantiles of interest for summarizing \code{stanfit} objects.}

\item{simplify}{Logical: Should the returned object be simplified by dropping the Monte Carlo standard error and the posterior standard deviation? Defaults to \code{TRUE}.}

\item{...}{Other arguments are passed on to \code{rstan::summary()} when summarizing \code{stanfit} objects.}
}
\value{
A tibble containing summaries of marginal posterior distributions. For objects produced by \code{fbam()}, only maximum a posteriori estimates are returned.
}
\description{
For objects produced by \code{hbam()}, this function is a wrapper for \code{rstan::summary()}. For objects produced by \code{fbam()} it offers a way to extract point estimates.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_data_cv.R
\name{prep_data_cv}
\alias{prep_data_cv}
\title{Prepare data for a K-fold cross-validation of an HBAM model}
\usage{
prep_data_cv(data, K = 10, seed = 1)
}
\arguments{
\item{data}{A list of data produced by \code{prep_data()}.}

\item{K}{An integer above 2, specifying the number of folds to use in the analysis. Defaults to 10.}

\item{seed}{An integer passed on to \code{set.seed} before creating the folds to increase reproducibility. Defaults to 1.}
}
\value{
A list of K data objects where each version includes a different vector identifying holdout-data.
}
\description{
This function turns data prepared for \code{hbam()} into a list of K versions, where each version includes a different vector identifying holdout-data.
}
\examples{
# Loading and re-coding ANES 1980 data:
data(LC1980)
LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA
self <- LC1980[, 1]
stimuli <- LC1980[, -1]
dat <- prep_data(self, stimuli)

# Prepare data for cross-validation:
dat_cv <- prep_data_cv(dat, K = 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hbam_cv.R
\name{hbam_cv}
\alias{hbam_cv}
\title{Perform K-fold cross-validation}
\usage{
hbam_cv(
  self = NULL,
  stimuli = NULL,
  model = "HBAM",
  allow_miss = 0,
  req_valid = NA,
  req_unique = 2,
  prefs = NULL,
  group_id = NULL,
  prep_data = TRUE,
  data = NULL,
  K = 10,
  chains = 2,
  warmup = 1000,
  iter = 3000,
  seed = 1,
  control = list(max_treedepth = 7),
  sigma_alpha = NULL,
  sigma_beta = 0.35,
  sigma_mu_alpha = NULL,
  sigma_mu_beta = 0.3,
  ...
)
}
\arguments{
\item{self}{A numerical vector of N ideological self-placements. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{stimuli}{An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{model}{Character: Name of the model to be used. Defaults to HBAM.}

\item{allow_miss}{Integer specifying how many missing stimulus positions should be accepted for an individual still to be included in the analysis. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function. Defaults to 0.}

\item{req_valid}{Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is \code{req_valid = J - allow_miss}, but if specified, \code{req_valid} takes precedence. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{req_unique}{Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is \code{req_unique = 2}. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{prefs}{An N × J matrix of numerical stimulus ratings or preference scores. These data are only required by the HBAM_R_MINI model and will be ignored when fitting other models.}

\item{group_id}{Integer vector of length N identifying which group each respondent belongs to. The supplied vector should range from 1 to the total number of groups in the data, and all integers between these numbers should be represented in the supplied data. These data are only required by models with "MULTI" in their name and will be ignored when fitting other models.}

\item{prep_data}{Logical: Should the data be prepared before fitting the model? (Or have the data been prepared in advance by first running the \code{prep_data()} and \code{prep_data_cv()} functions)? If so, set \code{prep_data = FALSE}.) Defaults to \code{prep_data = TRUE}.}

\item{data}{A list of data produced by \code{prep_data()} followed by \code{prep_data_cv()}.}

\item{K}{An integer above 2, specifying the number of folds to use in the analysis. Defaults to 10.}

\item{chains}{A positive integer specifying the number of Markov chains to use per fold. Defaults to 2.}

\item{warmup}{A positive integer specifying the number of warmup (aka burn-in) iterations per chain. It defaults to 1000. The number of warmup iterations should be smaller than \code{iter}.}

\item{iter}{A positive integer specifying the number of iterations for each chain (including warmup). It defaults to 3000 as running fewer chains for longer is a more efficient way to obtain a certain number of draws (and cross-validation can be computationally expensive).}

\item{seed}{An integer passed on to \code{set.seed} before creating the folds to increase reproducibility and comparability. Defaults to 1 and only applies to fold-creation when the argument \code{prep_data} is \code{TRUE}. The supplied \code{seed} argument is also used to generate seeds for the sampling algorithm.}

\item{control}{A named list of parameters to control the sampler's behavior. See the documentation for \code{rstan::stan} for more details.}

\item{sigma_alpha}{A positive numeric value specifying the standard deviation of the prior on the shift parameters in the FBAM model, or the standard deviation of the parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to B / 4, where B measures the length of the survey scale as the number of possible placements on one side of the center.}

\item{sigma_beta}{A positive numeric value specifying the standard deviation of the prior on the logged stretch parameters in the FBAM model, or the standard deviation of the logged parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to .35.}

\item{sigma_mu_alpha}{A positive numeric value specifying the standard deviation of the prior on the group-means of the shift parameters in MULTI-type models. Defaults to B / 5.}

\item{sigma_mu_beta}{A positive numeric value specifying the standard deviation of the prior on the group-means of the logged stretch parameters in MULTI-type models. Defaults to .3.}

\item{...}{Arguments passed to \code{rstan::sampling()}.}
}
\value{
A list of classes \code{kfold} and \code{loo}, which contains the following named elements:
\itemize{
\item \code{"estimates"}: A \verb{1x2} matrix containing the ELPD estimate and its standard error. The columns have names \code{"Estimate"} and \code{"SE"}.
\item \code{"pointwise"}: A \code{Nx1} matrix with column name \code{"elpd_kfold"} containing the pointwise contributions for each data point.
}
}
\description{
This function performs K-fold cross-validation for an HBAM or FBAM model in order to estimate the expected log pointwise predictive density for a new dataset (ELPD). Multiple chains for one or more folds can be run in parallel using the \code{future} package.
}
\examples{
\donttest{
# Loading and re-coding ANES 1980 data:
data(LC1980)
LC1980[LC1980 == 0 | LC1980 == 8 | LC1980 == 9] <- NA

# Making a small subset of the data for illustration:
self <- LC1980[1:50, 1]
stimuli <- LC1980[1:50, -1]

# Preparing to run chains in parallel using 2 cores via the future package:
  # Note: You would normally want to use all physical cores for this.
future::plan(future::multisession, workers = 2)

# Performing 4-fold cross-validation for the HBAM_MINI model:
  # Note: You would typically want to run the chains for more iterations.
cv_hbam_mini <- hbam_cv(self, stimuli, model = "HBAM_MINI", K = 4,
                        chains = 1, warmup = 500, iter = 1000)

# Performing 4-fold cross-validation for the FBAM model:
cv_fbam <- hbam_cv(self, stimuli, model = "FBAM", K = 4,
                        chains = 1, warmup = 500, iter = 1000)

# Comparing the results using the loo package:
loo::loo_compare(list(HBAM_MINI = cv_hbam_mini,
                 FBAM = cv_fbam))

# Stop the cluster of parallel sessions:
future::plan(future::sequential)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_plot_data.R
\name{get_plot_data}
\alias{get_plot_data}
\title{Extract data for plotting results from an HBAM model}
\usage{
get_plot_data(object, n_draws = 15, seed = 1)
}
\arguments{
\item{object}{An instance of class \code{stanfit} produced by \code{hbam()} or a list produced by \code{fbam()}.}

\item{n_draws}{Integer specifying the number of posterior draws to use when illustrating the uncertainty of the population distribution. This only applies for \code{stanfit} objects.}

\item{seed}{A positive integer specifying an optional seed for reproducibility. The seed is used to select respondent position draws for illustrating uncertainty. This only applies for \code{stanfit} objects.}
}
\value{
A list of three \code{tibble}s: The first element contains the posterior mean stimulus positions, as well as the x- and y-values of the posterior modes (which can be useful for labeling the distributions). The second element contains the posterior draws for the stimulus positions (which can be used to calculate marginal posterior densities). The third element contains the selected number of posterior draws for each respondent (which form the key ingredient for \code{plot_respondents}).
}
\description{
Extract data for plotting results from an HBAM model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_by_group.R
\name{plot_by_group}
\alias{plot_by_group}
\title{Plot posterior densities of parameter averages by group}
\usage{
plot_by_group(
  object,
  par = "abs_beta",
  group_id = NULL,
  ascending_means = TRUE,
  fill = "#2166AC",
  color = "#053061",
  alpha = 0.5,
  ncol = max(1, round(length(unique(group_id))/10))
)
}
\arguments{
\item{object}{An instance of class \code{stanfit} produced by \code{hbam()}.}

\item{par}{Character: Name of the parameter to be plotted. One of the following: \code{"alpha"}, \code{"beta"}, \code{"abs_beta"}, \code{"lambda"}, or \code{"chi"}. Defaults to \code{"abs_beta"}, which means the absolute values of the draws for beta will be used. Further individual-level parameters like \code{"eta"} can be specified if these have been passed to \code{hbam()} via the argument \code{extra_pars} when fitting the model. (Note that homoskedastic models have no \code{"eta"} parameters and "NF"-type models have no \code{"lambda"} or \code{"kappa"} parameters.)}

\item{group_id}{An optional vector that will be used to split the respondents into groups. The vector must either be as long as the number of rows in the original dataset, or as long as the number of respondents included in the analysis. If a \code{group_id} was previously supplied to \code{prep_data()} or \code{hbam()} and if no \code{group_id} is supplied here, the default is to use the existing \code{group_id}. If a \code{group_id} is supplied here, it will be used instead of any previously supplied vector. The \code{group_id} supplied here does not have to coincide with the \code{group_id} used to fit a "MULTI"-type model: Any vector that can be used to group the respondents is allowed.}

\item{ascending_means}{Logical: Should the groups be placed in ascending order based on their posterior means (\code{TRUE}) or should they be ordered based on their names (\code{FALSE})? Defaults to \code{TRUE}.}

\item{fill}{Fill color. Passed on to \code{ggplot2::geom_density()}.}

\item{color}{Color of outer lines. Passed on to \code{ggplot2::geom_density()}.}

\item{alpha}{Number in [0,1]: Inverse level of transparency.}

\item{ncol}{Number of columns. The default uses a formula to have approximately ten subplots per column.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot posterior densities of group summaries of individual parameters. The respondents can be grouped by any categorical variable and the function works whether the fitted model is of "MULTI"-type or not.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LC2012.R
\docType{data}
\name{LC2012}
\alias{LC2012}
\title{2012 Liberal-Conservative Scales}
\format{
An object of class \code{data.frame} with 5914 rows and 6 columns.
}
\source{
American National Election Studies.
}
\usage{
data(LC2012)
}
\description{
Liberal-Conservative 7-point scales from the 2012 National Election Study. Includes
(in order) original case id, self-placement, and rankings of Obama, Romney, Democratic Party,
Republican party. Missing values are coded as NA.
}
\examples{
data(LC2012)
head(LC2012)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fbam.R
\name{fbam}
\alias{fbam}
\title{Fit an FBAM model using optimization}
\usage{
fbam(
  self = NULL,
  stimuli = NULL,
  model = "FBAM",
  allow_miss = 2,
  req_valid = NA,
  req_unique = 2,
  group_id = NULL,
  data = NULL,
  seed = sample.int(.Machine$integer.max, 1),
  sigma_alpha = NULL,
  sigma_beta = 0.3,
  sigma_mu_alpha = NULL,
  sigma_mu_beta = 0.2,
  ...
)
}
\arguments{
\item{self}{An optional numerical vector of N ideological self-placements. Any missing data must be coded as NA. If this argument is not supplied (either here or in a previous call to \code{prep_data()}), respondent positions will not be estimated. If the data have been prepared in advance via the \code{prep_data()} function, the argument supplied here will be ignored.}

\item{stimuli}{An N × J matrix of numerical stimulus placements, where J is the number of stimuli. Any missing data must be coded as NA. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{model}{Character: Name of the model to be used. Defaults to FBAM. The available options are the three models with "FBAM" in their name. See the documentation for the \code{hbam()} function for descriptions of the models.}

\item{allow_miss}{Integer specifying how many missing stimulus positions to be accepted for an individual still to be included in the analysis. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function. Defaults to 2.}

\item{req_valid}{Integer specifying how many valid observations to require for a respondent to be included in the analysis. The default is \code{req_valid = J - allow_miss}, but if specified, \code{req_valid} takes precedence. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{req_unique}{Integer specifying how many unique positions on the ideological scale each respondent is required to have used when placing the stimuli in order to be included in the analysis. The default is \code{req_unique = 2}. This argument will not be used if the data have been prepared in advance via the \code{prep_data()} function.}

\item{group_id}{Vector of length N identifying which group each respondent belongs to. The format can be factor, character, integer, or numeric. Respondents with NAs on \code{group_id} will be dropped when \code{group_id} is supplied. These data are only required by models with \code{"MULTI"} in their name and will be ignored when fitting other models.}

\item{data}{List of data that have been prepared in advance via the \code{prep_data()} function. Not required if the arguments \code{self} and \code{stimuli} are provided.}

\item{seed}{A positive integer specifying an optional seed for reproducibility. If this argument is not supplied, a random seed will be generated and the function will produce slightly different results on each run.}

\item{sigma_alpha}{A positive numeric value specifying the standard deviation of the prior on the shift parameters in the FBAM model, or the standard deviation of the parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to B / 5, where B measures the length of the survey scale as the number of possible placements on one side of the center.}

\item{sigma_beta}{A positive numeric value specifying the standard deviation of the prior on the logged stretch parameters in the FBAM model, or the standard deviation of the logged parameters' deviation from the group-means in FBAM_MULTI models. (This argument will be ignored by HBAM models.) Defaults to .3.}

\item{sigma_mu_alpha}{A positive numeric value specifying the standard deviation of the prior on the group-means of the shift parameters in MULTI-type models. Defaults to B / 10.}

\item{sigma_mu_beta}{A positive numeric value specifying the standard deviation of the prior on the group-means of the logged stretch parameters in MULTI-type models. Defaults to .2.}

\item{...}{Arguments passed to \code{rstan::optimizing()}.}
}
\value{
A list produced by \code{rstan::optimizing()}.
}
\description{
Fit a simplified Bayesian Aldrich-McKelvey model with fixed hyperparameters using optimization via \code{rstan}. Users may replace the default priors by supplying their own values for the hyperparameters.
}
\examples{
\donttest{
# Loading ANES 2012 data:
data(LC2012)

# Making a small subset of the data for illustration:
self <- LC2012[1:1000, 2]
stimuli <- LC2012[1:1000, -c(1:2)]

# Fitting the FBAM model:
fit_fbam <- fbam(self, stimuli)

# Obtaining point estimates for the latent stimulus positions:
theta_est <- get_est(fit_fbam, par = "theta")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_respondents.R
\name{plot_respondents}
\alias{plot_respondents}
\title{Plot estimated respondent positions}
\usage{
plot_respondents(
  object,
  inc_stimuli = TRUE,
  n_draws = 15,
  color = "#053061",
  fill = "#2166AC",
  alpha_color = 0.6,
  alpha_fill = 0.7/n_draws,
  seed = 1
)
}
\arguments{
\item{object}{An instance of class \code{stanfit} produced by \code{hbam()}.}

\item{inc_stimuli}{Logical: Should estimated stimulus positions also be shown?}

\item{n_draws}{Integer specifying the number of posterior draws to use when illustrating the uncertainty of the population distribution. Defaults to 15.}

\item{color}{Color of lines illustrating uncertainty.}

\item{fill}{Fill color for density plots.}

\item{alpha_color}{Number in [0,1]: Inverse level of transparency for line color.}

\item{alpha_fill}{Number in [0,1]: Inverse level of transparency for fill color.}

\item{seed}{A positive integer specifying an optional seed for reproducibility. The seed is used to select respondent position draws for illustrating uncertainty.}
}
\value{
A \code{ggplot} object.
}
\description{
Plot the distribution of estimated respondent positions from an HBAM model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_over_self.R
\name{plot_over_self}
\alias{plot_over_self}
\title{Plot individual parameter estimates over self-placements}
\usage{
plot_over_self(
  object,
  par = "chi",
  estimate = "median",
  names = NULL,
  parlabel = NULL,
  fill = "#2166AC",
  color = "#053061",
  width = 0.7,
  alpha = 0.5,
  outlier.size = 0.3,
  median_color = "black",
  median_lwd = 0.7
)
}
\arguments{
\item{object}{An object of class \code{stanfit} produced by \code{hbam()}, a list produced by \code{fbam()}, or a \code{list} of such objects, which will produce a faceted plot.}

\item{par}{Character: Name of the parameter to be plotted. One of the following: \code{"alpha"}, \code{"beta"}, \code{"abs_beta"}, \code{"lambda"}, or \code{"chi"}. Defaults to \code{"chi"}. Further individual-level parameters like \code{"eta"} can be specified if these have been passed to \code{hbam()} via the argument \code{extra_pars} when fitting the model. (Note that homoskedastic models have no \code{"eta"} parameters and "NF"-type models have no \code{"lambda"} or \code{"kappa"} parameters.)}

\item{estimate}{Character: Specifying which type of posterior point estimate to use. One of \code{"median"} and \code{"mean"}. Defaults to \code{"median"}. This only applies for \code{stanfit} objects.}

\item{names}{An optional character vector of model names of same length as the supplied list of models.}

\item{parlabel}{An optional character containing an alternative label for the parameter (will be parsed if passed as an expression).}

\item{fill}{Fill color of boxes. Passed on to \code{ggplot2::geom_boxplot()}.}

\item{color}{Color of outer lines. Passed on to \code{ggplot2::geom_boxplot()}.}

\item{width}{Width of boxes. Passed on to \code{ggplot2::geom_boxplot()}.}

\item{alpha}{Number in [0,1]: Inverse level of transparency for fill color.}

\item{outlier.size}{Size of dots representing outliers. Passed on to \code{ggplot2::geom_boxplot()}.}

\item{median_color}{Color of solid line representing the median.}

\item{median_lwd}{Thickness of solid line representing the median.}
}
\value{
A \code{ggplot} object.
}
\description{
Create a boxplot of individual parameter point estimates from an HBAM model over self-placements
}
